# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: openshell.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""

from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC, 6, 31, 1, "", "openshell.proto"
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import datamodel_pb2 as datamodel__pb2
from . import sandbox_pb2 as sandbox__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x0fopenshell.proto\x12\x0copenshell.v1\x1a\x0f\x64\x61tamodel.proto\x1a\rsandbox.proto"\x0f\n\rHealthRequest"N\n\x0eHealthResponse\x12+\n\x06status\x18\x01 \x01(\x0e\x32\x1b.openshell.v1.ServiceStatus\x12\x0f\n\x07version\x18\x02 \x01(\t"W\n\x14\x43reateSandboxRequest\x12\x31\n\x04spec\x18\x01 \x01(\x0b\x32#.openshell.datamodel.v1.SandboxSpec\x12\x0c\n\x04name\x18\x02 \x01(\t"!\n\x11GetSandboxRequest\x12\x0c\n\x04name\x18\x01 \x01(\t"5\n\x14ListSandboxesRequest\x12\r\n\x05limit\x18\x01 \x01(\r\x12\x0e\n\x06offset\x18\x02 \x01(\r"$\n\x14\x44\x65leteSandboxRequest\x12\x0c\n\x04name\x18\x01 \x01(\t"C\n\x0fSandboxResponse\x12\x30\n\x07sandbox\x18\x01 \x01(\x0b\x32\x1f.openshell.datamodel.v1.Sandbox"K\n\x15ListSandboxesResponse\x12\x32\n\tsandboxes\x18\x01 \x03(\x0b\x32\x1f.openshell.datamodel.v1.Sandbox"(\n\x15\x44\x65leteSandboxResponse\x12\x0f\n\x07\x64\x65leted\x18\x01 \x01(\x08"-\n\x17\x43reateSshSessionRequest\x12\x12\n\nsandbox_id\x18\x01 \x01(\t"\xcc\x01\n\x18\x43reateSshSessionResponse\x12\x12\n\nsandbox_id\x18\x01 \x01(\t\x12\r\n\x05token\x18\x02 \x01(\t\x12\x14\n\x0cgateway_host\x18\x03 \x01(\t\x12\x14\n\x0cgateway_port\x18\x04 \x01(\r\x12\x16\n\x0egateway_scheme\x18\x05 \x01(\t\x12\x14\n\x0c\x63onnect_path\x18\x06 \x01(\t\x12\x1c\n\x14host_key_fingerprint\x18\x07 \x01(\t\x12\x15\n\rexpires_at_ms\x18\x08 \x01(\x03"(\n\x17RevokeSshSessionRequest\x12\r\n\x05token\x18\x01 \x01(\t"+\n\x18RevokeSshSessionResponse\x12\x0f\n\x07revoked\x18\x01 \x01(\x08"\xee\x01\n\x12\x45xecSandboxRequest\x12\x12\n\nsandbox_id\x18\x01 \x01(\t\x12\x0f\n\x07\x63ommand\x18\x02 \x03(\t\x12\x0f\n\x07workdir\x18\x03 \x01(\t\x12\x46\n\x0b\x65nvironment\x18\x04 \x03(\x0b\x32\x31.openshell.v1.ExecSandboxRequest.EnvironmentEntry\x12\x17\n\x0ftimeout_seconds\x18\x05 \x01(\r\x12\r\n\x05stdin\x18\x06 \x01(\x0c\x1a\x32\n\x10\x45nvironmentEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01"!\n\x11\x45xecSandboxStdout\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\x0c"!\n\x11\x45xecSandboxStderr\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\x0c"$\n\x0f\x45xecSandboxExit\x12\x11\n\texit_code\x18\x01 \x01(\x05"\xb2\x01\n\x10\x45xecSandboxEvent\x12\x31\n\x06stdout\x18\x01 \x01(\x0b\x32\x1f.openshell.v1.ExecSandboxStdoutH\x00\x12\x31\n\x06stderr\x18\x02 \x01(\x0b\x32\x1f.openshell.v1.ExecSandboxStderrH\x00\x12-\n\x04\x65xit\x18\x03 \x01(\x0b\x32\x1d.openshell.v1.ExecSandboxExitH\x00\x42\t\n\x07payload"\x88\x01\n\nSshSession\x12\n\n\x02id\x18\x01 \x01(\t\x12\x12\n\nsandbox_id\x18\x02 \x01(\t\x12\r\n\x05token\x18\x03 \x01(\t\x12\x15\n\rcreated_at_ms\x18\x04 \x01(\x03\x12\x0f\n\x07revoked\x18\x05 \x01(\x08\x12\x0c\n\x04name\x18\x06 \x01(\t\x12\x15\n\rexpires_at_ms\x18\x07 \x01(\x03"\xec\x01\n\x13WatchSandboxRequest\x12\n\n\x02id\x18\x01 \x01(\t\x12\x15\n\rfollow_status\x18\x02 \x01(\x08\x12\x13\n\x0b\x66ollow_logs\x18\x03 \x01(\x08\x12\x15\n\rfollow_events\x18\x04 \x01(\x08\x12\x16\n\x0elog_tail_lines\x18\x05 \x01(\r\x12\x12\n\nevent_tail\x18\x06 \x01(\r\x12\x18\n\x10stop_on_terminal\x18\x07 \x01(\x08\x12\x14\n\x0clog_since_ms\x18\x08 \x01(\x03\x12\x13\n\x0blog_sources\x18\t \x03(\t\x12\x15\n\rlog_min_level\x18\n \x01(\t"\xa5\x02\n\x12SandboxStreamEvent\x12\x32\n\x07sandbox\x18\x01 \x01(\x0b\x32\x1f.openshell.datamodel.v1.SandboxH\x00\x12+\n\x03log\x18\x02 \x01(\x0b\x32\x1c.openshell.v1.SandboxLogLineH\x00\x12,\n\x05\x65vent\x18\x03 \x01(\x0b\x32\x1b.openshell.v1.PlatformEventH\x00\x12\x35\n\x07warning\x18\x04 \x01(\x0b\x32".openshell.v1.SandboxStreamWarningH\x00\x12>\n\x13\x64raft_policy_update\x18\x05 \x01(\x0b\x32\x1f.openshell.v1.DraftPolicyUpdateH\x00\x42\t\n\x07payload"\xe3\x01\n\x0eSandboxLogLine\x12\x12\n\nsandbox_id\x18\x01 \x01(\t\x12\x14\n\x0ctimestamp_ms\x18\x02 \x01(\x03\x12\r\n\x05level\x18\x03 \x01(\t\x12\x0e\n\x06target\x18\x04 \x01(\t\x12\x0f\n\x07message\x18\x05 \x01(\t\x12\x0e\n\x06source\x18\x06 \x01(\t\x12\x38\n\x06\x66ields\x18\x07 \x03(\x0b\x32(.openshell.v1.SandboxLogLine.FieldsEntry\x1a-\n\x0b\x46ieldsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01"\xd2\x01\n\rPlatformEvent\x12\x14\n\x0ctimestamp_ms\x18\x01 \x01(\x03\x12\x0e\n\x06source\x18\x02 \x01(\t\x12\x0c\n\x04type\x18\x03 \x01(\t\x12\x0e\n\x06reason\x18\x04 \x01(\t\x12\x0f\n\x07message\x18\x05 \x01(\t\x12;\n\x08metadata\x18\x06 \x03(\x0b\x32).openshell.v1.PlatformEvent.MetadataEntry\x1a/\n\rMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01"\'\n\x14SandboxStreamWarning\x12\x0f\n\x07message\x18\x01 \x01(\t"K\n\x15\x43reateProviderRequest\x12\x32\n\x08provider\x18\x01 \x01(\x0b\x32 .openshell.datamodel.v1.Provider""\n\x12GetProviderRequest\x12\x0c\n\x04name\x18\x01 \x01(\t"5\n\x14ListProvidersRequest\x12\r\n\x05limit\x18\x01 \x01(\r\x12\x0e\n\x06offset\x18\x02 \x01(\r"K\n\x15UpdateProviderRequest\x12\x32\n\x08provider\x18\x01 \x01(\x0b\x32 .openshell.datamodel.v1.Provider"%\n\x15\x44\x65leteProviderRequest\x12\x0c\n\x04name\x18\x01 \x01(\t"F\n\x10ProviderResponse\x12\x32\n\x08provider\x18\x01 \x01(\x0b\x32 .openshell.datamodel.v1.Provider"L\n\x15ListProvidersResponse\x12\x33\n\tproviders\x18\x01 \x03(\x0b\x32 .openshell.datamodel.v1.Provider")\n\x16\x44\x65leteProviderResponse\x12\x0f\n\x07\x64\x65leted\x18\x01 \x01(\x08":\n$GetSandboxProviderEnvironmentRequest\x12\x12\n\nsandbox_id\x18\x01 \x01(\t"\xb6\x01\n%GetSandboxProviderEnvironmentResponse\x12Y\n\x0b\x65nvironment\x18\x01 \x03(\x0b\x32\x44.openshell.v1.GetSandboxProviderEnvironmentResponse.EnvironmentEntry\x1a\x32\n\x10\x45nvironmentEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01"\xd0\x01\n\x13UpdateConfigRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x33\n\x06policy\x18\x02 \x01(\x0b\x32#.openshell.sandbox.v1.SandboxPolicy\x12\x13\n\x0bsetting_key\x18\x03 \x01(\t\x12\x39\n\rsetting_value\x18\x04 \x01(\x0b\x32".openshell.sandbox.v1.SettingValue\x12\x16\n\x0e\x64\x65lete_setting\x18\x05 \x01(\x08\x12\x0e\n\x06global\x18\x06 \x01(\x08"h\n\x14UpdateConfigResponse\x12\x0f\n\x07version\x18\x01 \x01(\r\x12\x13\n\x0bpolicy_hash\x18\x02 \x01(\t\x12\x19\n\x11settings_revision\x18\x03 \x01(\x04\x12\x0f\n\x07\x64\x65leted\x18\x04 \x01(\x08"N\n\x1dGetSandboxPolicyStatusRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07version\x18\x02 \x01(\r\x12\x0e\n\x06global\x18\x03 \x01(\x08"o\n\x1eGetSandboxPolicyStatusResponse\x12\x35\n\x08revision\x18\x01 \x01(\x0b\x32#.openshell.v1.SandboxPolicyRevision\x12\x16\n\x0e\x61\x63tive_version\x18\x02 \x01(\r"Y\n\x1aListSandboxPoliciesRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\r\n\x05limit\x18\x02 \x01(\r\x12\x0e\n\x06offset\x18\x03 \x01(\r\x12\x0e\n\x06global\x18\x04 \x01(\x08"U\n\x1bListSandboxPoliciesResponse\x12\x36\n\trevisions\x18\x01 \x03(\x0b\x32#.openshell.v1.SandboxPolicyRevision"\x80\x01\n\x19ReportPolicyStatusRequest\x12\x12\n\nsandbox_id\x18\x01 \x01(\t\x12\x0f\n\x07version\x18\x02 \x01(\r\x12*\n\x06status\x18\x03 \x01(\x0e\x32\x1a.openshell.v1.PolicyStatus\x12\x12\n\nload_error\x18\x04 \x01(\t"\x1c\n\x1aReportPolicyStatusResponse"\xdf\x01\n\x15SandboxPolicyRevision\x12\x0f\n\x07version\x18\x01 \x01(\r\x12\x13\n\x0bpolicy_hash\x18\x02 \x01(\t\x12*\n\x06status\x18\x03 \x01(\x0e\x32\x1a.openshell.v1.PolicyStatus\x12\x12\n\nload_error\x18\x04 \x01(\t\x12\x15\n\rcreated_at_ms\x18\x05 \x01(\x03\x12\x14\n\x0cloaded_at_ms\x18\x06 \x01(\x03\x12\x33\n\x06policy\x18\x07 \x01(\x0b\x32#.openshell.sandbox.v1.SandboxPolicy"p\n\x15GetSandboxLogsRequest\x12\x12\n\nsandbox_id\x18\x01 \x01(\t\x12\r\n\x05lines\x18\x02 \x01(\r\x12\x10\n\x08since_ms\x18\x03 \x01(\x03\x12\x0f\n\x07sources\x18\x04 \x03(\t\x12\x11\n\tmin_level\x18\x05 \x01(\t"X\n\x16PushSandboxLogsRequest\x12\x12\n\nsandbox_id\x18\x01 \x01(\t\x12*\n\x04logs\x18\x02 \x03(\x0b\x32\x1c.openshell.v1.SandboxLogLine"\x19\n\x17PushSandboxLogsResponse"Z\n\x16GetSandboxLogsResponse\x12*\n\x04logs\x18\x01 \x03(\x0b\x32\x1c.openshell.v1.SandboxLogLine\x12\x14\n\x0c\x62uffer_total\x18\x02 \x01(\r"P\n\x0fL7RequestSample\x12\x0e\n\x06method\x18\x01 \x01(\t\x12\x0c\n\x04path\x18\x02 \x01(\t\x12\x10\n\x08\x64\x65\x63ision\x18\x03 \x01(\t\x12\r\n\x05\x63ount\x18\x04 \x01(\r"\x95\x03\n\rDenialSummary\x12\x12\n\nsandbox_id\x18\x01 \x01(\t\x12\x0c\n\x04host\x18\x02 \x01(\t\x12\x0c\n\x04port\x18\x03 \x01(\r\x12\x0e\n\x06\x62inary\x18\x04 \x01(\t\x12\x11\n\tancestors\x18\x05 \x03(\t\x12\x13\n\x0b\x64\x65ny_reason\x18\x06 \x01(\t\x12\x15\n\rfirst_seen_ms\x18\x07 \x01(\x03\x12\x14\n\x0clast_seen_ms\x18\x08 \x01(\x03\x12\r\n\x05\x63ount\x18\t \x01(\r\x12\x18\n\x10suppressed_count\x18\n \x01(\r\x12\x13\n\x0btotal_count\x18\x0b \x01(\r\x12\x17\n\x0fsample_cmdlines\x18\x0c \x03(\t\x12\x15\n\rbinary_sha256\x18\r \x01(\t\x12\x12\n\npersistent\x18\x0e \x01(\x08\x12\x14\n\x0c\x64\x65nial_stage\x18\x0f \x01(\t\x12\x39\n\x12l7_request_samples\x18\x10 \x03(\x0b\x32\x1d.openshell.v1.L7RequestSample\x12\x1c\n\x14l7_inspection_active\x18\x11 \x01(\x08"\x81\x03\n\x0bPolicyChunk\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0e\n\x06status\x18\x02 \x01(\t\x12\x11\n\trule_name\x18\x03 \x01(\t\x12>\n\rproposed_rule\x18\x04 \x01(\x0b\x32\'.openshell.sandbox.v1.NetworkPolicyRule\x12\x11\n\trationale\x18\x05 \x01(\t\x12\x16\n\x0esecurity_notes\x18\x06 \x01(\t\x12\x12\n\nconfidence\x18\x07 \x01(\x02\x12\x1a\n\x12\x64\x65nial_summary_ids\x18\x08 \x03(\t\x12\x15\n\rcreated_at_ms\x18\t \x01(\x03\x12\x15\n\rdecided_at_ms\x18\n \x01(\x03\x12\r\n\x05stage\x18\x0b \x01(\t\x12\x1b\n\x13supersedes_chunk_id\x18\x0c \x01(\t\x12\x11\n\thit_count\x18\r \x01(\x05\x12\x15\n\rfirst_seen_ms\x18\x0e \x01(\x03\x12\x14\n\x0clast_seen_ms\x18\x0f \x01(\x03\x12\x0e\n\x06\x62inary\x18\x10 \x01(\t"f\n\x11\x44raftPolicyUpdate\x12\x15\n\rdraft_version\x18\x01 \x01(\x04\x12\x12\n\nnew_chunks\x18\x02 \x01(\r\x12\x15\n\rtotal_pending\x18\x03 \x01(\r\x12\x0f\n\x07summary\x18\x04 \x01(\t"\xa6\x01\n\x1bSubmitPolicyAnalysisRequest\x12.\n\tsummaries\x18\x01 \x03(\x0b\x32\x1b.openshell.v1.DenialSummary\x12\x32\n\x0fproposed_chunks\x18\x02 \x03(\x0b\x32\x19.openshell.v1.PolicyChunk\x12\x15\n\ranalysis_mode\x18\x03 \x01(\t\x12\x0c\n\x04name\x18\x04 \x01(\t"k\n\x1cSubmitPolicyAnalysisResponse\x12\x17\n\x0f\x61\x63\x63\x65pted_chunks\x18\x01 \x01(\r\x12\x17\n\x0frejected_chunks\x18\x02 \x01(\r\x12\x19\n\x11rejection_reasons\x18\x03 \x03(\t"<\n\x15GetDraftPolicyRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x15\n\rstatus_filter\x18\x02 \x01(\t"\x90\x01\n\x16GetDraftPolicyResponse\x12)\n\x06\x63hunks\x18\x01 \x03(\x0b\x32\x19.openshell.v1.PolicyChunk\x12\x17\n\x0frolling_summary\x18\x02 \x01(\t\x12\x15\n\rdraft_version\x18\x03 \x01(\x04\x12\x1b\n\x13last_analyzed_at_ms\x18\x04 \x01(\x03":\n\x18\x41pproveDraftChunkRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x10\n\x08\x63hunk_id\x18\x02 \x01(\t"H\n\x19\x41pproveDraftChunkResponse\x12\x16\n\x0epolicy_version\x18\x01 \x01(\r\x12\x13\n\x0bpolicy_hash\x18\x02 \x01(\t"I\n\x17RejectDraftChunkRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x10\n\x08\x63hunk_id\x18\x02 \x01(\t\x12\x0e\n\x06reason\x18\x03 \x01(\t"\x1a\n\x18RejectDraftChunkResponse"N\n\x1c\x41pproveAllDraftChunksRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12 \n\x18include_security_flagged\x18\x02 \x01(\x08"}\n\x1d\x41pproveAllDraftChunksResponse\x12\x16\n\x0epolicy_version\x18\x01 \x01(\r\x12\x13\n\x0bpolicy_hash\x18\x02 \x01(\t\x12\x17\n\x0f\x63hunks_approved\x18\x03 \x01(\r\x12\x16\n\x0e\x63hunks_skipped\x18\x04 \x01(\r"w\n\x15\x45\x64itDraftChunkRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x10\n\x08\x63hunk_id\x18\x02 \x01(\t\x12>\n\rproposed_rule\x18\x03 \x01(\x0b\x32\'.openshell.sandbox.v1.NetworkPolicyRule"\x18\n\x16\x45\x64itDraftChunkResponse"7\n\x15UndoDraftChunkRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x10\n\x08\x63hunk_id\x18\x02 \x01(\t"E\n\x16UndoDraftChunkResponse\x12\x16\n\x0epolicy_version\x18\x01 \x01(\r\x12\x13\n\x0bpolicy_hash\x18\x02 \x01(\t"\'\n\x17\x43learDraftChunksRequest\x12\x0c\n\x04name\x18\x01 \x01(\t"2\n\x18\x43learDraftChunksResponse\x12\x16\n\x0e\x63hunks_cleared\x18\x01 \x01(\r"&\n\x16GetDraftHistoryRequest\x12\x0c\n\x04name\x18\x01 \x01(\t"d\n\x11\x44raftHistoryEntry\x12\x14\n\x0ctimestamp_ms\x18\x01 \x01(\x03\x12\x12\n\nevent_type\x18\x02 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x03 \x01(\t\x12\x10\n\x08\x63hunk_id\x18\x04 \x01(\t"K\n\x17GetDraftHistoryResponse\x12\x30\n\x07\x65ntries\x18\x01 \x03(\x0b\x32\x1f.openshell.v1.DraftHistoryEntry*\x9a\x01\n\x0cPolicyStatus\x12\x1d\n\x19POLICY_STATUS_UNSPECIFIED\x10\x00\x12\x19\n\x15POLICY_STATUS_PENDING\x10\x01\x12\x18\n\x14POLICY_STATUS_LOADED\x10\x02\x12\x18\n\x14POLICY_STATUS_FAILED\x10\x03\x12\x1c\n\x18POLICY_STATUS_SUPERSEDED\x10\x04*\x86\x01\n\rServiceStatus\x12\x1e\n\x1aSERVICE_STATUS_UNSPECIFIED\x10\x00\x12\x1a\n\x16SERVICE_STATUS_HEALTHY\x10\x01\x12\x1b\n\x17SERVICE_STATUS_DEGRADED\x10\x02\x12\x1c\n\x18SERVICE_STATUS_UNHEALTHY\x10\x03\x32\xa1\x18\n\tOpenShell\x12\x43\n\x06Health\x12\x1b.openshell.v1.HealthRequest\x1a\x1c.openshell.v1.HealthResponse\x12R\n\rCreateSandbox\x12".openshell.v1.CreateSandboxRequest\x1a\x1d.openshell.v1.SandboxResponse\x12L\n\nGetSandbox\x12\x1f.openshell.v1.GetSandboxRequest\x1a\x1d.openshell.v1.SandboxResponse\x12X\n\rListSandboxes\x12".openshell.v1.ListSandboxesRequest\x1a#.openshell.v1.ListSandboxesResponse\x12X\n\rDeleteSandbox\x12".openshell.v1.DeleteSandboxRequest\x1a#.openshell.v1.DeleteSandboxResponse\x12\x61\n\x10\x43reateSshSession\x12%.openshell.v1.CreateSshSessionRequest\x1a&.openshell.v1.CreateSshSessionResponse\x12\x61\n\x10RevokeSshSession\x12%.openshell.v1.RevokeSshSessionRequest\x1a&.openshell.v1.RevokeSshSessionResponse\x12Q\n\x0b\x45xecSandbox\x12 .openshell.v1.ExecSandboxRequest\x1a\x1e.openshell.v1.ExecSandboxEvent0\x01\x12U\n\x0e\x43reateProvider\x12#.openshell.v1.CreateProviderRequest\x1a\x1e.openshell.v1.ProviderResponse\x12O\n\x0bGetProvider\x12 .openshell.v1.GetProviderRequest\x1a\x1e.openshell.v1.ProviderResponse\x12X\n\rListProviders\x12".openshell.v1.ListProvidersRequest\x1a#.openshell.v1.ListProvidersResponse\x12U\n\x0eUpdateProvider\x12#.openshell.v1.UpdateProviderRequest\x1a\x1e.openshell.v1.ProviderResponse\x12[\n\x0e\x44\x65leteProvider\x12#.openshell.v1.DeleteProviderRequest\x1a$.openshell.v1.DeleteProviderResponse\x12q\n\x10GetSandboxConfig\x12-.openshell.sandbox.v1.GetSandboxConfigRequest\x1a..openshell.sandbox.v1.GetSandboxConfigResponse\x12q\n\x10GetGatewayConfig\x12-.openshell.sandbox.v1.GetGatewayConfigRequest\x1a..openshell.sandbox.v1.GetGatewayConfigResponse\x12U\n\x0cUpdateConfig\x12!.openshell.v1.UpdateConfigRequest\x1a".openshell.v1.UpdateConfigResponse\x12s\n\x16GetSandboxPolicyStatus\x12+.openshell.v1.GetSandboxPolicyStatusRequest\x1a,.openshell.v1.GetSandboxPolicyStatusResponse\x12j\n\x13ListSandboxPolicies\x12(.openshell.v1.ListSandboxPoliciesRequest\x1a).openshell.v1.ListSandboxPoliciesResponse\x12g\n\x12ReportPolicyStatus\x12\'.openshell.v1.ReportPolicyStatusRequest\x1a(.openshell.v1.ReportPolicyStatusResponse\x12\x88\x01\n\x1dGetSandboxProviderEnvironment\x12\x32.openshell.v1.GetSandboxProviderEnvironmentRequest\x1a\x33.openshell.v1.GetSandboxProviderEnvironmentResponse\x12[\n\x0eGetSandboxLogs\x12#.openshell.v1.GetSandboxLogsRequest\x1a$.openshell.v1.GetSandboxLogsResponse\x12`\n\x0fPushSandboxLogs\x12$.openshell.v1.PushSandboxLogsRequest\x1a%.openshell.v1.PushSandboxLogsResponse(\x01\x12U\n\x0cWatchSandbox\x12!.openshell.v1.WatchSandboxRequest\x1a .openshell.v1.SandboxStreamEvent0\x01\x12m\n\x14SubmitPolicyAnalysis\x12).openshell.v1.SubmitPolicyAnalysisRequest\x1a*.openshell.v1.SubmitPolicyAnalysisResponse\x12[\n\x0eGetDraftPolicy\x12#.openshell.v1.GetDraftPolicyRequest\x1a$.openshell.v1.GetDraftPolicyResponse\x12\x64\n\x11\x41pproveDraftChunk\x12&.openshell.v1.ApproveDraftChunkRequest\x1a\'.openshell.v1.ApproveDraftChunkResponse\x12\x61\n\x10RejectDraftChunk\x12%.openshell.v1.RejectDraftChunkRequest\x1a&.openshell.v1.RejectDraftChunkResponse\x12p\n\x15\x41pproveAllDraftChunks\x12*.openshell.v1.ApproveAllDraftChunksRequest\x1a+.openshell.v1.ApproveAllDraftChunksResponse\x12[\n\x0e\x45\x64itDraftChunk\x12#.openshell.v1.EditDraftChunkRequest\x1a$.openshell.v1.EditDraftChunkResponse\x12[\n\x0eUndoDraftChunk\x12#.openshell.v1.UndoDraftChunkRequest\x1a$.openshell.v1.UndoDraftChunkResponse\x12\x61\n\x10\x43learDraftChunks\x12%.openshell.v1.ClearDraftChunksRequest\x1a&.openshell.v1.ClearDraftChunksResponse\x12^\n\x0fGetDraftHistory\x12$.openshell.v1.GetDraftHistoryRequest\x1a%.openshell.v1.GetDraftHistoryResponseB\x1e\n\x1a\x63om.anthropic.openshell.v1P\x01\x62\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "openshell_pb2", _globals)
if not _descriptor._USE_C_DESCRIPTORS:
    _globals["DESCRIPTOR"]._loaded_options = None
    _globals["DESCRIPTOR"]._serialized_options = b"\n\032com.anthropic.openshell.v1P\001"
    _globals["_EXECSANDBOXREQUEST_ENVIRONMENTENTRY"]._loaded_options = None
    _globals["_EXECSANDBOXREQUEST_ENVIRONMENTENTRY"]._serialized_options = b"8\001"
    _globals["_SANDBOXLOGLINE_FIELDSENTRY"]._loaded_options = None
    _globals["_SANDBOXLOGLINE_FIELDSENTRY"]._serialized_options = b"8\001"
    _globals["_PLATFORMEVENT_METADATAENTRY"]._loaded_options = None
    _globals["_PLATFORMEVENT_METADATAENTRY"]._serialized_options = b"8\001"
    _globals["_GETSANDBOXPROVIDERENVIRONMENTRESPONSE_ENVIRONMENTENTRY"]._loaded_options = None
    _globals[
        "_GETSANDBOXPROVIDERENVIRONMENTRESPONSE_ENVIRONMENTENTRY"
    ]._serialized_options = b"8\001"
    _globals["_POLICYSTATUS"]._serialized_start = 7217
    _globals["_POLICYSTATUS"]._serialized_end = 7371
    _globals["_SERVICESTATUS"]._serialized_start = 7374
    _globals["_SERVICESTATUS"]._serialized_end = 7508
    _globals["_HEALTHREQUEST"]._serialized_start = 65
    _globals["_HEALTHREQUEST"]._serialized_end = 80
    _globals["_HEALTHRESPONSE"]._serialized_start = 82
    _globals["_HEALTHRESPONSE"]._serialized_end = 160
    _globals["_CREATESANDBOXREQUEST"]._serialized_start = 162
    _globals["_CREATESANDBOXREQUEST"]._serialized_end = 249
    _globals["_GETSANDBOXREQUEST"]._serialized_start = 251
    _globals["_GETSANDBOXREQUEST"]._serialized_end = 284
    _globals["_LISTSANDBOXESREQUEST"]._serialized_start = 286
    _globals["_LISTSANDBOXESREQUEST"]._serialized_end = 339
    _globals["_DELETESANDBOXREQUEST"]._serialized_start = 341
    _globals["_DELETESANDBOXREQUEST"]._serialized_end = 377
    _globals["_SANDBOXRESPONSE"]._serialized_start = 379
    _globals["_SANDBOXRESPONSE"]._serialized_end = 446
    _globals["_LISTSANDBOXESRESPONSE"]._serialized_start = 448
    _globals["_LISTSANDBOXESRESPONSE"]._serialized_end = 523
    _globals["_DELETESANDBOXRESPONSE"]._serialized_start = 525
    _globals["_DELETESANDBOXRESPONSE"]._serialized_end = 565
    _globals["_CREATESSHSESSIONREQUEST"]._serialized_start = 567
    _globals["_CREATESSHSESSIONREQUEST"]._serialized_end = 612
    _globals["_CREATESSHSESSIONRESPONSE"]._serialized_start = 615
    _globals["_CREATESSHSESSIONRESPONSE"]._serialized_end = 819
    _globals["_REVOKESSHSESSIONREQUEST"]._serialized_start = 821
    _globals["_REVOKESSHSESSIONREQUEST"]._serialized_end = 861
    _globals["_REVOKESSHSESSIONRESPONSE"]._serialized_start = 863
    _globals["_REVOKESSHSESSIONRESPONSE"]._serialized_end = 906
    _globals["_EXECSANDBOXREQUEST"]._serialized_start = 909
    _globals["_EXECSANDBOXREQUEST"]._serialized_end = 1147
    _globals["_EXECSANDBOXREQUEST_ENVIRONMENTENTRY"]._serialized_start = 1097
    _globals["_EXECSANDBOXREQUEST_ENVIRONMENTENTRY"]._serialized_end = 1147
    _globals["_EXECSANDBOXSTDOUT"]._serialized_start = 1149
    _globals["_EXECSANDBOXSTDOUT"]._serialized_end = 1182
    _globals["_EXECSANDBOXSTDERR"]._serialized_start = 1184
    _globals["_EXECSANDBOXSTDERR"]._serialized_end = 1217
    _globals["_EXECSANDBOXEXIT"]._serialized_start = 1219
    _globals["_EXECSANDBOXEXIT"]._serialized_end = 1255
    _globals["_EXECSANDBOXEVENT"]._serialized_start = 1258
    _globals["_EXECSANDBOXEVENT"]._serialized_end = 1436
    _globals["_SSHSESSION"]._serialized_start = 1439
    _globals["_SSHSESSION"]._serialized_end = 1575
    _globals["_WATCHSANDBOXREQUEST"]._serialized_start = 1578
    _globals["_WATCHSANDBOXREQUEST"]._serialized_end = 1814
    _globals["_SANDBOXSTREAMEVENT"]._serialized_start = 1817
    _globals["_SANDBOXSTREAMEVENT"]._serialized_end = 2110
    _globals["_SANDBOXLOGLINE"]._serialized_start = 2113
    _globals["_SANDBOXLOGLINE"]._serialized_end = 2340
    _globals["_SANDBOXLOGLINE_FIELDSENTRY"]._serialized_start = 2295
    _globals["_SANDBOXLOGLINE_FIELDSENTRY"]._serialized_end = 2340
    _globals["_PLATFORMEVENT"]._serialized_start = 2343
    _globals["_PLATFORMEVENT"]._serialized_end = 2553
    _globals["_PLATFORMEVENT_METADATAENTRY"]._serialized_start = 2506
    _globals["_PLATFORMEVENT_METADATAENTRY"]._serialized_end = 2553
    _globals["_SANDBOXSTREAMWARNING"]._serialized_start = 2555
    _globals["_SANDBOXSTREAMWARNING"]._serialized_end = 2594
    _globals["_CREATEPROVIDERREQUEST"]._serialized_start = 2596
    _globals["_CREATEPROVIDERREQUEST"]._serialized_end = 2671
    _globals["_GETPROVIDERREQUEST"]._serialized_start = 2673
    _globals["_GETPROVIDERREQUEST"]._serialized_end = 2707
    _globals["_LISTPROVIDERSREQUEST"]._serialized_start = 2709
    _globals["_LISTPROVIDERSREQUEST"]._serialized_end = 2762
    _globals["_UPDATEPROVIDERREQUEST"]._serialized_start = 2764
    _globals["_UPDATEPROVIDERREQUEST"]._serialized_end = 2839
    _globals["_DELETEPROVIDERREQUEST"]._serialized_start = 2841
    _globals["_DELETEPROVIDERREQUEST"]._serialized_end = 2878
    _globals["_PROVIDERRESPONSE"]._serialized_start = 2880
    _globals["_PROVIDERRESPONSE"]._serialized_end = 2950
    _globals["_LISTPROVIDERSRESPONSE"]._serialized_start = 2952
    _globals["_LISTPROVIDERSRESPONSE"]._serialized_end = 3028
    _globals["_DELETEPROVIDERRESPONSE"]._serialized_start = 3030
    _globals["_DELETEPROVIDERRESPONSE"]._serialized_end = 3071
    _globals["_GETSANDBOXPROVIDERENVIRONMENTREQUEST"]._serialized_start = 3073
    _globals["_GETSANDBOXPROVIDERENVIRONMENTREQUEST"]._serialized_end = 3131
    _globals["_GETSANDBOXPROVIDERENVIRONMENTRESPONSE"]._serialized_start = 3134
    _globals["_GETSANDBOXPROVIDERENVIRONMENTRESPONSE"]._serialized_end = 3316
    _globals["_GETSANDBOXPROVIDERENVIRONMENTRESPONSE_ENVIRONMENTENTRY"]._serialized_start = 1097
    _globals["_GETSANDBOXPROVIDERENVIRONMENTRESPONSE_ENVIRONMENTENTRY"]._serialized_end = 1147
    _globals["_UPDATECONFIGREQUEST"]._serialized_start = 3319
    _globals["_UPDATECONFIGREQUEST"]._serialized_end = 3527
    _globals["_UPDATECONFIGRESPONSE"]._serialized_start = 3529
    _globals["_UPDATECONFIGRESPONSE"]._serialized_end = 3633
    _globals["_GETSANDBOXPOLICYSTATUSREQUEST"]._serialized_start = 3635
    _globals["_GETSANDBOXPOLICYSTATUSREQUEST"]._serialized_end = 3713
    _globals["_GETSANDBOXPOLICYSTATUSRESPONSE"]._serialized_start = 3715
    _globals["_GETSANDBOXPOLICYSTATUSRESPONSE"]._serialized_end = 3826
    _globals["_LISTSANDBOXPOLICIESREQUEST"]._serialized_start = 3828
    _globals["_LISTSANDBOXPOLICIESREQUEST"]._serialized_end = 3917
    _globals["_LISTSANDBOXPOLICIESRESPONSE"]._serialized_start = 3919
    _globals["_LISTSANDBOXPOLICIESRESPONSE"]._serialized_end = 4004
    _globals["_REPORTPOLICYSTATUSREQUEST"]._serialized_start = 4007
    _globals["_REPORTPOLICYSTATUSREQUEST"]._serialized_end = 4135
    _globals["_REPORTPOLICYSTATUSRESPONSE"]._serialized_start = 4137
    _globals["_REPORTPOLICYSTATUSRESPONSE"]._serialized_end = 4165
    _globals["_SANDBOXPOLICYREVISION"]._serialized_start = 4168
    _globals["_SANDBOXPOLICYREVISION"]._serialized_end = 4391
    _globals["_GETSANDBOXLOGSREQUEST"]._serialized_start = 4393
    _globals["_GETSANDBOXLOGSREQUEST"]._serialized_end = 4505
    _globals["_PUSHSANDBOXLOGSREQUEST"]._serialized_start = 4507
    _globals["_PUSHSANDBOXLOGSREQUEST"]._serialized_end = 4595
    _globals["_PUSHSANDBOXLOGSRESPONSE"]._serialized_start = 4597
    _globals["_PUSHSANDBOXLOGSRESPONSE"]._serialized_end = 4622
    _globals["_GETSANDBOXLOGSRESPONSE"]._serialized_start = 4624
    _globals["_GETSANDBOXLOGSRESPONSE"]._serialized_end = 4714
    _globals["_L7REQUESTSAMPLE"]._serialized_start = 4716
    _globals["_L7REQUESTSAMPLE"]._serialized_end = 4796
    _globals["_DENIALSUMMARY"]._serialized_start = 4799
    _globals["_DENIALSUMMARY"]._serialized_end = 5204
    _globals["_POLICYCHUNK"]._serialized_start = 5207
    _globals["_POLICYCHUNK"]._serialized_end = 5592
    _globals["_DRAFTPOLICYUPDATE"]._serialized_start = 5594
    _globals["_DRAFTPOLICYUPDATE"]._serialized_end = 5696
    _globals["_SUBMITPOLICYANALYSISREQUEST"]._serialized_start = 5699
    _globals["_SUBMITPOLICYANALYSISREQUEST"]._serialized_end = 5865
    _globals["_SUBMITPOLICYANALYSISRESPONSE"]._serialized_start = 5867
    _globals["_SUBMITPOLICYANALYSISRESPONSE"]._serialized_end = 5974
    _globals["_GETDRAFTPOLICYREQUEST"]._serialized_start = 5976
    _globals["_GETDRAFTPOLICYREQUEST"]._serialized_end = 6036
    _globals["_GETDRAFTPOLICYRESPONSE"]._serialized_start = 6039
    _globals["_GETDRAFTPOLICYRESPONSE"]._serialized_end = 6183
    _globals["_APPROVEDRAFTCHUNKREQUEST"]._serialized_start = 6185
    _globals["_APPROVEDRAFTCHUNKREQUEST"]._serialized_end = 6243
    _globals["_APPROVEDRAFTCHUNKRESPONSE"]._serialized_start = 6245
    _globals["_APPROVEDRAFTCHUNKRESPONSE"]._serialized_end = 6317
    _globals["_REJECTDRAFTCHUNKREQUEST"]._serialized_start = 6319
    _globals["_REJECTDRAFTCHUNKREQUEST"]._serialized_end = 6392
    _globals["_REJECTDRAFTCHUNKRESPONSE"]._serialized_start = 6394
    _globals["_REJECTDRAFTCHUNKRESPONSE"]._serialized_end = 6420
    _globals["_APPROVEALLDRAFTCHUNKSREQUEST"]._serialized_start = 6422
    _globals["_APPROVEALLDRAFTCHUNKSREQUEST"]._serialized_end = 6500
    _globals["_APPROVEALLDRAFTCHUNKSRESPONSE"]._serialized_start = 6502
    _globals["_APPROVEALLDRAFTCHUNKSRESPONSE"]._serialized_end = 6627
    _globals["_EDITDRAFTCHUNKREQUEST"]._serialized_start = 6629
    _globals["_EDITDRAFTCHUNKREQUEST"]._serialized_end = 6748
    _globals["_EDITDRAFTCHUNKRESPONSE"]._serialized_start = 6750
    _globals["_EDITDRAFTCHUNKRESPONSE"]._serialized_end = 6774
    _globals["_UNDODRAFTCHUNKREQUEST"]._serialized_start = 6776
    _globals["_UNDODRAFTCHUNKREQUEST"]._serialized_end = 6831
    _globals["_UNDODRAFTCHUNKRESPONSE"]._serialized_start = 6833
    _globals["_UNDODRAFTCHUNKRESPONSE"]._serialized_end = 6902
    _globals["_CLEARDRAFTCHUNKSREQUEST"]._serialized_start = 6904
    _globals["_CLEARDRAFTCHUNKSREQUEST"]._serialized_end = 6943
    _globals["_CLEARDRAFTCHUNKSRESPONSE"]._serialized_start = 6945
    _globals["_CLEARDRAFTCHUNKSRESPONSE"]._serialized_end = 6995
    _globals["_GETDRAFTHISTORYREQUEST"]._serialized_start = 6997
    _globals["_GETDRAFTHISTORYREQUEST"]._serialized_end = 7035
    _globals["_DRAFTHISTORYENTRY"]._serialized_start = 7037
    _globals["_DRAFTHISTORYENTRY"]._serialized_end = 7137
    _globals["_GETDRAFTHISTORYRESPONSE"]._serialized_start = 7139
    _globals["_GETDRAFTHISTORYRESPONSE"]._serialized_end = 7214
    _globals["_OPENSHELL"]._serialized_start = 7511
    _globals["_OPENSHELL"]._serialized_end = 10616
# @@protoc_insertion_point(module_scope)



"""Shared dependencies for API routes.

Uses a ContextVar to resolve the gateway name per request,
so route handlers don't need to accept a gateway parameter.
"""

from __future__ import annotations

import logging
from contextvars import ContextVar
from typing import TYPE_CHECKING

from fastapi import HTTPException, Request

from shoreguard.client import ShoreGuardClient
from shoreguard.config import VALID_GATEWAY_NAME_RE

if TYPE_CHECKING:
    from shoreguard.services.gateway import GatewayService

logger = logging.getLogger(__name__)

_current_gateway: ContextVar[str | None] = ContextVar("_current_gateway", default=None)

_VALID_GW_RE = VALID_GATEWAY_NAME_RE


def _get_gateway_service() -> GatewayService:
    """Return the global gateway service singleton.

    Raises:
        HTTPException: If the gateway service has not been initialised.

    Returns:
        GatewayService: The active gateway service instance.
    """
    from shoreguard.services.gateway import gateway_service

    if gateway_service is None:
        raise HTTPException(503, "GatewayService not initialised — app lifespan has not started")
    return gateway_service


def get_actor(request: Request) -> str:
    """Extract the acting user identity from the request state.

    Args:
        request: The incoming HTTP request.

    Returns:
        str: The user identity string, or ``"unknown"`` if not set.
    """
    return getattr(request.state, "user_id", "unknown")


def resolve_gateway(gw: str) -> None:
    """FastAPI dependency — set the gateway context for this request.

    Args:
        gw: The gateway name from the URL path.

    Raises:
        HTTPException: If the gateway name does not match the allowed pattern.
    """
    if not _VALID_GW_RE.match(gw):
        raise HTTPException(400, "Invalid gateway name: must match [a-zA-Z0-9][a-zA-Z0-9._-]*")
    logger.debug("Resolved gateway context: '%s'", gw)
    _current_gateway.set(gw)


def _require_gateway_name() -> str:
    """Return the current gateway name, raising if not set.

    Returns:
        str: The gateway name from the request context.

    Raises:
        HTTPException: If no gateway context has been set.
    """
    gw = _current_gateway.get()
    if gw is None:
        raise HTTPException(500, "No gateway context — resolve_gateway dependency missing")
    return gw


def get_client() -> ShoreGuardClient:
    """Return a client for the current gateway (from ContextVar or active config).

    Returns:
        ShoreGuardClient: The client bound to the current gateway context.
    """
    return _get_gateway_service().get_client(name=_require_gateway_name())


def set_client(client: ShoreGuardClient | None) -> None:
    """Set or clear a client for the current gateway.

    Args:
        client: The client instance to set, or ``None`` to clear.
    """
    _get_gateway_service().set_client(client, name=_require_gateway_name())


def reset_backoff() -> None:
    """Reset the connection backoff for the current gateway."""
    _get_gateway_service().reset_backoff(name=_require_gateway_name())



from . import datamodel_pb2 as _datamodel_pb2
from . import sandbox_pb2 as _sandbox_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from collections.abc import Iterable as _Iterable, Mapping as _Mapping
from typing import ClassVar as _ClassVar, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class PolicyStatus(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    POLICY_STATUS_UNSPECIFIED: _ClassVar[PolicyStatus]
    POLICY_STATUS_PENDING: _ClassVar[PolicyStatus]
    POLICY_STATUS_LOADED: _ClassVar[PolicyStatus]
    POLICY_STATUS_FAILED: _ClassVar[PolicyStatus]
    POLICY_STATUS_SUPERSEDED: _ClassVar[PolicyStatus]

class ServiceStatus(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    SERVICE_STATUS_UNSPECIFIED: _ClassVar[ServiceStatus]
    SERVICE_STATUS_HEALTHY: _ClassVar[ServiceStatus]
    SERVICE_STATUS_DEGRADED: _ClassVar[ServiceStatus]
    SERVICE_STATUS_UNHEALTHY: _ClassVar[ServiceStatus]

POLICY_STATUS_UNSPECIFIED: PolicyStatus
POLICY_STATUS_PENDING: PolicyStatus
POLICY_STATUS_LOADED: PolicyStatus
POLICY_STATUS_FAILED: PolicyStatus
POLICY_STATUS_SUPERSEDED: PolicyStatus
SERVICE_STATUS_UNSPECIFIED: ServiceStatus
SERVICE_STATUS_HEALTHY: ServiceStatus
SERVICE_STATUS_DEGRADED: ServiceStatus
SERVICE_STATUS_UNHEALTHY: ServiceStatus

class HealthRequest(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class HealthResponse(_message.Message):
    __slots__ = ("status", "version")
    STATUS_FIELD_NUMBER: _ClassVar[int]
    VERSION_FIELD_NUMBER: _ClassVar[int]
    status: ServiceStatus
    version: str
    def __init__(
        self, status: _Optional[_Union[ServiceStatus, str]] = ..., version: _Optional[str] = ...
    ) -> None: ...

class CreateSandboxRequest(_message.Message):
    __slots__ = ("spec", "name")
    SPEC_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    spec: _datamodel_pb2.SandboxSpec
    name: str
    def __init__(
        self,
        spec: _Optional[_Union[_datamodel_pb2.SandboxSpec, _Mapping]] = ...,
        name: _Optional[str] = ...,
    ) -> None: ...

class GetSandboxRequest(_message.Message):
    __slots__ = ("name",)
    NAME_FIELD_NUMBER: _ClassVar[int]
    name: str
    def __init__(self, name: _Optional[str] = ...) -> None: ...

class ListSandboxesRequest(_message.Message):
    __slots__ = ("limit", "offset")
    LIMIT_FIELD_NUMBER: _ClassVar[int]
    OFFSET_FIELD_NUMBER: _ClassVar[int]
    limit: int
    offset: int
    def __init__(self, limit: _Optional[int] = ..., offset: _Optional[int] = ...) -> None: ...

class DeleteSandboxRequest(_message.Message):
    __slots__ = ("name",)
    NAME_FIELD_NUMBER: _ClassVar[int]
    name: str
    def __init__(self, name: _Optional[str] = ...) -> None: ...

class SandboxResponse(_message.Message):
    __slots__ = ("sandbox",)
    SANDBOX_FIELD_NUMBER: _ClassVar[int]
    sandbox: _datamodel_pb2.Sandbox
    def __init__(
        self, sandbox: _Optional[_Union[_datamodel_pb2.Sandbox, _Mapping]] = ...
    ) -> None: ...

class ListSandboxesResponse(_message.Message):
    __slots__ = ("sandboxes",)
    SANDBOXES_FIELD_NUMBER: _ClassVar[int]
    sandboxes: _containers.RepeatedCompositeFieldContainer[_datamodel_pb2.Sandbox]
    def __init__(
        self, sandboxes: _Optional[_Iterable[_Union[_datamodel_pb2.Sandbox, _Mapping]]] = ...
    ) -> None: ...

class DeleteSandboxResponse(_message.Message):
    __slots__ = ("deleted",)
    DELETED_FIELD_NUMBER: _ClassVar[int]
    deleted: bool
    def __init__(self, deleted: bool = ...) -> None: ...

class CreateSshSessionRequest(_message.Message):
    __slots__ = ("sandbox_id",)
    SANDBOX_ID_FIELD_NUMBER: _ClassVar[int]
    sandbox_id: str
    def __init__(self, sandbox_id: _Optional[str] = ...) -> None: ...

class CreateSshSessionResponse(_message.Message):
    __slots__ = (
        "sandbox_id",
        "token",
        "gateway_host",
        "gateway_port",
        "gateway_scheme",
        "connect_path",
        "host_key_fingerprint",
        "expires_at_ms",
    )
    SANDBOX_ID_FIELD_NUMBER: _ClassVar[int]
    TOKEN_FIELD_NUMBER: _ClassVar[int]
    GATEWAY_HOST_FIELD_NUMBER: _ClassVar[int]
    GATEWAY_PORT_FIELD_NUMBER: _ClassVar[int]
    GATEWAY_SCHEME_FIELD_NUMBER: _ClassVar[int]
    CONNECT_PATH_FIELD_NUMBER: _ClassVar[int]
    HOST_KEY_FINGERPRINT_FIELD_NUMBER: _ClassVar[int]
    EXPIRES_AT_MS_FIELD_NUMBER: _ClassVar[int]
    sandbox_id: str
    token: str
    gateway_host: str
    gateway_port: int
    gateway_scheme: str
    connect_path: str
    host_key_fingerprint: str
    expires_at_ms: int
    def __init__(
        self,
        sandbox_id: _Optional[str] = ...,
        token: _Optional[str] = ...,
        gateway_host: _Optional[str] = ...,
        gateway_port: _Optional[int] = ...,
        gateway_scheme: _Optional[str] = ...,
        connect_path: _Optional[str] = ...,
        host_key_fingerprint: _Optional[str] = ...,
        expires_at_ms: _Optional[int] = ...,
    ) -> None: ...

class RevokeSshSessionRequest(_message.Message):
    __slots__ = ("token",)
    TOKEN_FIELD_NUMBER: _ClassVar[int]
    token: str
    def __init__(self, token: _Optional[str] = ...) -> None: ...

class RevokeSshSessionResponse(_message.Message):
    __slots__ = ("revoked",)
    REVOKED_FIELD_NUMBER: _ClassVar[int]
    revoked: bool
    def __init__(self, revoked: bool = ...) -> None: ...

class ExecSandboxRequest(_message.Message):
    __slots__ = ("sandbox_id", "command", "workdir", "environment", "timeout_seconds", "stdin")
    class EnvironmentEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: str
        def __init__(self, key: _Optional[str] = ..., value: _Optional[str] = ...) -> None: ...

    SANDBOX_ID_FIELD_NUMBER: _ClassVar[int]
    COMMAND_FIELD_NUMBER: _ClassVar[int]
    WORKDIR_FIELD_NUMBER: _ClassVar[int]
    ENVIRONMENT_FIELD_NUMBER: _ClassVar[int]
    TIMEOUT_SECONDS_FIELD_NUMBER: _ClassVar[int]
    STDIN_FIELD_NUMBER: _ClassVar[int]
    sandbox_id: str
    command: _containers.RepeatedScalarFieldContainer[str]
    workdir: str
    environment: _containers.ScalarMap[str, str]
    timeout_seconds: int
    stdin: bytes
    def __init__(
        self,
        sandbox_id: _Optional[str] = ...,
        command: _Optional[_Iterable[str]] = ...,
        workdir: _Optional[str] = ...,
        environment: _Optional[_Mapping[str, str]] = ...,
        timeout_seconds: _Optional[int] = ...,
        stdin: _Optional[bytes] = ...,
    ) -> None: ...

class ExecSandboxStdout(_message.Message):
    __slots__ = ("data",)
    DATA_FIELD_NUMBER: _ClassVar[int]
    data: bytes
    def __init__(self, data: _Optional[bytes] = ...) -> None: ...

class ExecSandboxStderr(_message.Message):
    __slots__ = ("data",)
    DATA_FIELD_NUMBER: _ClassVar[int]
    data: bytes
    def __init__(self, data: _Optional[bytes] = ...) -> None: ...

class ExecSandboxExit(_message.Message):
    __slots__ = ("exit_code",)
    EXIT_CODE_FIELD_NUMBER: _ClassVar[int]
    exit_code: int
    def __init__(self, exit_code: _Optional[int] = ...) -> None: ...

class ExecSandboxEvent(_message.Message):
    __slots__ = ("stdout", "stderr", "exit")
    STDOUT_FIELD_NUMBER: _ClassVar[int]
    STDERR_FIELD_NUMBER: _ClassVar[int]
    EXIT_FIELD_NUMBER: _ClassVar[int]
    stdout: ExecSandboxStdout
    stderr: ExecSandboxStderr
    exit: ExecSandboxExit
    def __init__(
        self,
        stdout: _Optional[_Union[ExecSandboxStdout, _Mapping]] = ...,
        stderr: _Optional[_Union[ExecSandboxStderr, _Mapping]] = ...,
        exit: _Optional[_Union[ExecSandboxExit, _Mapping]] = ...,
    ) -> None: ...

class SshSession(_message.Message):
    __slots__ = ("id", "sandbox_id", "token", "created_at_ms", "revoked", "name", "expires_at_ms")
    ID_FIELD_NUMBER: _ClassVar[int]
    SANDBOX_ID_FIELD_NUMBER: _ClassVar[int]
    TOKEN_FIELD_NUMBER: _ClassVar[int]
    CREATED_AT_MS_FIELD_NUMBER: _ClassVar[int]
    REVOKED_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    EXPIRES_AT_MS_FIELD_NUMBER: _ClassVar[int]
    id: str
    sandbox_id: str
    token: str
    created_at_ms: int
    revoked: bool
    name: str
    expires_at_ms: int
    def __init__(
        self,
        id: _Optional[str] = ...,
        sandbox_id: _Optional[str] = ...,
        token: _Optional[str] = ...,
        created_at_ms: _Optional[int] = ...,
        revoked: bool = ...,
        name: _Optional[str] = ...,
        expires_at_ms: _Optional[int] = ...,
    ) -> None: ...

class WatchSandboxRequest(_message.Message):
    __slots__ = (
        "id",
        "follow_status",
        "follow_logs",
        "follow_events",
        "log_tail_lines",
        "event_tail",
        "stop_on_terminal",
        "log_since_ms",
        "log_sources",
        "log_min_level",
    )
    ID_FIELD_NUMBER: _ClassVar[int]
    FOLLOW_STATUS_FIELD_NUMBER: _ClassVar[int]
    FOLLOW_LOGS_FIELD_NUMBER: _ClassVar[int]
    FOLLOW_EVENTS_FIELD_NUMBER: _ClassVar[int]
    LOG_TAIL_LINES_FIELD_NUMBER: _ClassVar[int]
    EVENT_TAIL_FIELD_NUMBER: _ClassVar[int]
    STOP_ON_TERMINAL_FIELD_NUMBER: _ClassVar[int]
    LOG_SINCE_MS_FIELD_NUMBER: _ClassVar[int]
    LOG_SOURCES_FIELD_NUMBER: _ClassVar[int]
    LOG_MIN_LEVEL_FIELD_NUMBER: _ClassVar[int]
    id: str
    follow_status: bool
    follow_logs: bool
    follow_events: bool
    log_tail_lines: int
    event_tail: int
    stop_on_terminal: bool
    log_since_ms: int
    log_sources: _containers.RepeatedScalarFieldContainer[str]
    log_min_level: str
    def __init__(
        self,
        id: _Optional[str] = ...,
        follow_status: bool = ...,
        follow_logs: bool = ...,
        follow_events: bool = ...,
        log_tail_lines: _Optional[int] = ...,
        event_tail: _Optional[int] = ...,
        stop_on_terminal: bool = ...,
        log_since_ms: _Optional[int] = ...,
        log_sources: _Optional[_Iterable[str]] = ...,
        log_min_level: _Optional[str] = ...,
    ) -> None: ...

class SandboxStreamEvent(_message.Message):
    __slots__ = ("sandbox", "log", "event", "warning", "draft_policy_update")
    SANDBOX_FIELD_NUMBER: _ClassVar[int]
    LOG_FIELD_NUMBER: _ClassVar[int]
    EVENT_FIELD_NUMBER: _ClassVar[int]
    WARNING_FIELD_NUMBER: _ClassVar[int]
    DRAFT_POLICY_UPDATE_FIELD_NUMBER: _ClassVar[int]
    sandbox: _datamodel_pb2.Sandbox
    log: SandboxLogLine
    event: PlatformEvent
    warning: SandboxStreamWarning
    draft_policy_update: DraftPolicyUpdate
    def __init__(
        self,
        sandbox: _Optional[_Union[_datamodel_pb2.Sandbox, _Mapping]] = ...,
        log: _Optional[_Union[SandboxLogLine, _Mapping]] = ...,
        event: _Optional[_Union[PlatformEvent, _Mapping]] = ...,
        warning: _Optional[_Union[SandboxStreamWarning, _Mapping]] = ...,
        draft_policy_update: _Optional[_Union[DraftPolicyUpdate, _Mapping]] = ...,
    ) -> None: ...

class SandboxLogLine(_message.Message):
    __slots__ = ("sandbox_id", "timestamp_ms", "level", "target", "message", "source", "fields")
    class FieldsEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: str
        def __init__(self, key: _Optional[str] = ..., value: _Optional[str] = ...) -> None: ...

    SANDBOX_ID_FIELD_NUMBER: _ClassVar[int]
    TIMESTAMP_MS_FIELD_NUMBER: _ClassVar[int]
    LEVEL_FIELD_NUMBER: _ClassVar[int]
    TARGET_FIELD_NUMBER: _ClassVar[int]
    MESSAGE_FIELD_NUMBER: _ClassVar[int]
    SOURCE_FIELD_NUMBER: _ClassVar[int]
    FIELDS_FIELD_NUMBER: _ClassVar[int]
    sandbox_id: str
    timestamp_ms: int
    level: str
    target: str
    message: str
    source: str
    fields: _containers.ScalarMap[str, str]
    def __init__(
        self,
        sandbox_id: _Optional[str] = ...,
        timestamp_ms: _Optional[int] = ...,
        level: _Optional[str] = ...,
        target: _Optional[str] = ...,
        message: _Optional[str] = ...,
        source: _Optional[str] = ...,
        fields: _Optional[_Mapping[str, str]] = ...,
    ) -> None: ...

class PlatformEvent(_message.Message):
    __slots__ = ("timestamp_ms", "source", "type", "reason", "message", "metadata")
    class MetadataEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: str
        def __init__(self, key: _Optional[str] = ..., value: _Optional[str] = ...) -> None: ...

    TIMESTAMP_MS_FIELD_NUMBER: _ClassVar[int]
    SOURCE_FIELD_NUMBER: _ClassVar[int]
    TYPE_FIELD_NUMBER: _ClassVar[int]
    REASON_FIELD_NUMBER: _ClassVar[int]
    MESSAGE_FIELD_NUMBER: _ClassVar[int]
    METADATA_FIELD_NUMBER: _ClassVar[int]
    timestamp_ms: int
    source: str
    type: str
    reason: str
    message: str
    metadata: _containers.ScalarMap[str, str]
    def __init__(
        self,
        timestamp_ms: _Optional[int] = ...,
        source: _Optional[str] = ...,
        type: _Optional[str] = ...,
        reason: _Optional[str] = ...,
        message: _Optional[str] = ...,
        metadata: _Optional[_Mapping[str, str]] = ...,
    ) -> None: ...

class SandboxStreamWarning(_message.Message):
    __slots__ = ("message",)
    MESSAGE_FIELD_NUMBER: _ClassVar[int]
    message: str
    def __init__(self, message: _Optional[str] = ...) -> None: ...

class CreateProviderRequest(_message.Message):
    __slots__ = ("provider",)
    PROVIDER_FIELD_NUMBER: _ClassVar[int]
    provider: _datamodel_pb2.Provider
    def __init__(
        self, provider: _Optional[_Union[_datamodel_pb2.Provider, _Mapping]] = ...
    ) -> None: ...

class GetProviderRequest(_message.Message):
    __slots__ = ("name",)
    NAME_FIELD_NUMBER: _ClassVar[int]
    name: str
    def __init__(self, name: _Optional[str] = ...) -> None: ...

class ListProvidersRequest(_message.Message):
    __slots__ = ("limit", "offset")
    LIMIT_FIELD_NUMBER: _ClassVar[int]
    OFFSET_FIELD_NUMBER: _ClassVar[int]
    limit: int
    offset: int
    def __init__(self, limit: _Optional[int] = ..., offset: _Optional[int] = ...) -> None: ...

class UpdateProviderRequest(_message.Message):
    __slots__ = ("provider",)
    PROVIDER_FIELD_NUMBER: _ClassVar[int]
    provider: _datamodel_pb2.Provider
    def __init__(
        self, provider: _Optional[_Union[_datamodel_pb2.Provider, _Mapping]] = ...
    ) -> None: ...

class DeleteProviderRequest(_message.Message):
    __slots__ = ("name",)
    NAME_FIELD_NUMBER: _ClassVar[int]
    name: str
    def __init__(self, name: _Optional[str] = ...) -> None: ...

class ProviderResponse(_message.Message):
    __slots__ = ("provider",)
    PROVIDER_FIELD_NUMBER: _ClassVar[int]
    provider: _datamodel_pb2.Provider
    def __init__(
        self, provider: _Optional[_Union[_datamodel_pb2.Provider, _Mapping]] = ...
    ) -> None: ...

class ListProvidersResponse(_message.Message):
    __slots__ = ("providers",)
    PROVIDERS_FIELD_NUMBER: _ClassVar[int]
    providers: _containers.RepeatedCompositeFieldContainer[_datamodel_pb2.Provider]
    def __init__(
        self, providers: _Optional[_Iterable[_Union[_datamodel_pb2.Provider, _Mapping]]] = ...
    ) -> None: ...

class DeleteProviderResponse(_message.Message):
    __slots__ = ("deleted",)
    DELETED_FIELD_NUMBER: _ClassVar[int]
    deleted: bool
    def __init__(self, deleted: bool = ...) -> None: ...

class GetSandboxProviderEnvironmentRequest(_message.Message):
    __slots__ = ("sandbox_id",)
    SANDBOX_ID_FIELD_NUMBER: _ClassVar[int]
    sandbox_id: str
    def __init__(self, sandbox_id: _Optional[str] = ...) -> None: ...

class GetSandboxProviderEnvironmentResponse(_message.Message):
    __slots__ = ("environment",)
    class EnvironmentEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: str
        def __init__(self, key: _Optional[str] = ..., value: _Optional[str] = ...) -> None: ...

    ENVIRONMENT_FIELD_NUMBER: _ClassVar[int]
    environment: _containers.ScalarMap[str, str]
    def __init__(self, environment: _Optional[_Mapping[str, str]] = ...) -> None: ...

class UpdateConfigRequest(_message.Message):
    __slots__ = ("name", "policy", "setting_key", "setting_value", "delete_setting")
    NAME_FIELD_NUMBER: _ClassVar[int]
    POLICY_FIELD_NUMBER: _ClassVar[int]
    SETTING_KEY_FIELD_NUMBER: _ClassVar[int]
    SETTING_VALUE_FIELD_NUMBER: _ClassVar[int]
    DELETE_SETTING_FIELD_NUMBER: _ClassVar[int]
    GLOBAL_FIELD_NUMBER: _ClassVar[int]
    name: str
    policy: _sandbox_pb2.SandboxPolicy
    setting_key: str
    setting_value: _sandbox_pb2.SettingValue
    delete_setting: bool
    def __init__(
        self,
        name: _Optional[str] = ...,
        policy: _Optional[_Union[_sandbox_pb2.SandboxPolicy, _Mapping]] = ...,
        setting_key: _Optional[str] = ...,
        setting_value: _Optional[_Union[_sandbox_pb2.SettingValue, _Mapping]] = ...,
        delete_setting: bool = ...,
        **kwargs,
    ) -> None: ...

class UpdateConfigResponse(_message.Message):
    __slots__ = ("version", "policy_hash", "settings_revision", "deleted")
    VERSION_FIELD_NUMBER: _ClassVar[int]
    POLICY_HASH_FIELD_NUMBER: _ClassVar[int]
    SETTINGS_REVISION_FIELD_NUMBER: _ClassVar[int]
    DELETED_FIELD_NUMBER: _ClassVar[int]
    version: int
    policy_hash: str
    settings_revision: int
    deleted: bool
    def __init__(
        self,
        version: _Optional[int] = ...,
        policy_hash: _Optional[str] = ...,
        settings_revision: _Optional[int] = ...,
        deleted: bool = ...,
    ) -> None: ...

class GetSandboxPolicyStatusRequest(_message.Message):
    __slots__ = ("name", "version")
    NAME_FIELD_NUMBER: _ClassVar[int]
    VERSION_FIELD_NUMBER: _ClassVar[int]
    GLOBAL_FIELD_NUMBER: _ClassVar[int]
    name: str
    version: int
    def __init__(
        self, name: _Optional[str] = ..., version: _Optional[int] = ..., **kwargs
    ) -> None: ...

class GetSandboxPolicyStatusResponse(_message.Message):
    __slots__ = ("revision", "active_version")
    REVISION_FIELD_NUMBER: _ClassVar[int]
    ACTIVE_VERSION_FIELD_NUMBER: _ClassVar[int]
    revision: SandboxPolicyRevision
    active_version: int
    def __init__(
        self,
        revision: _Optional[_Union[SandboxPolicyRevision, _Mapping]] = ...,
        active_version: _Optional[int] = ...,
    ) -> None: ...

class ListSandboxPoliciesRequest(_message.Message):
    __slots__ = ("name", "limit", "offset")
    NAME_FIELD_NUMBER: _ClassVar[int]
    LIMIT_FIELD_NUMBER: _ClassVar[int]
    OFFSET_FIELD_NUMBER: _ClassVar[int]
    GLOBAL_FIELD_NUMBER: _ClassVar[int]
    name: str
    limit: int
    offset: int
    def __init__(
        self,
        name: _Optional[str] = ...,
        limit: _Optional[int] = ...,
        offset: _Optional[int] = ...,
        **kwargs,
    ) -> None: ...

class ListSandboxPoliciesResponse(_message.Message):
    __slots__ = ("revisions",)
    REVISIONS_FIELD_NUMBER: _ClassVar[int]
    revisions: _containers.RepeatedCompositeFieldContainer[SandboxPolicyRevision]
    def __init__(
        self, revisions: _Optional[_Iterable[_Union[SandboxPolicyRevision, _Mapping]]] = ...
    ) -> None: ...

class ReportPolicyStatusRequest(_message.Message):
    __slots__ = ("sandbox_id", "version", "status", "load_error")
    SANDBOX_ID_FIELD_NUMBER: _ClassVar[int]
    VERSION_FIELD_NUMBER: _ClassVar[int]
    STATUS_FIELD_NUMBER: _ClassVar[int]
    LOAD_ERROR_FIELD_NUMBER: _ClassVar[int]
    sandbox_id: str
    version: int
    status: PolicyStatus
    load_error: str
    def __init__(
        self,
        sandbox_id: _Optional[str] = ...,
        version: _Optional[int] = ...,
        status: _Optional[_Union[PolicyStatus, str]] = ...,
        load_error: _Optional[str] = ...,
    ) -> None: ...

class ReportPolicyStatusResponse(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class SandboxPolicyRevision(_message.Message):
    __slots__ = (
        "version",
        "policy_hash",
        "status",
        "load_error",
        "created_at_ms",
        "loaded_at_ms",
        "policy",
    )
    VERSION_FIELD_NUMBER: _ClassVar[int]
    POLICY_HASH_FIELD_NUMBER: _ClassVar[int]
    STATUS_FIELD_NUMBER: _ClassVar[int]
    LOAD_ERROR_FIELD_NUMBER: _ClassVar[int]
    CREATED_AT_MS_FIELD_NUMBER: _ClassVar[int]
    LOADED_AT_MS_FIELD_NUMBER: _ClassVar[int]
    POLICY_FIELD_NUMBER: _ClassVar[int]
    version: int
    policy_hash: str
    status: PolicyStatus
    load_error: str
    created_at_ms: int
    loaded_at_ms: int
    policy: _sandbox_pb2.SandboxPolicy
    def __init__(
        self,
        version: _Optional[int] = ...,
        policy_hash: _Optional[str] = ...,
        status: _Optional[_Union[PolicyStatus, str]] = ...,
        load_error: _Optional[str] = ...,
        created_at_ms: _Optional[int] = ...,
        loaded_at_ms: _Optional[int] = ...,
        policy: _Optional[_Union[_sandbox_pb2.SandboxPolicy, _Mapping]] = ...,
    ) -> None: ...

class GetSandboxLogsRequest(_message.Message):
    __slots__ = ("sandbox_id", "lines", "since_ms", "sources", "min_level")
    SANDBOX_ID_FIELD_NUMBER: _ClassVar[int]
    LINES_FIELD_NUMBER: _ClassVar[int]
    SINCE_MS_FIELD_NUMBER: _ClassVar[int]
    SOURCES_FIELD_NUMBER: _ClassVar[int]
    MIN_LEVEL_FIELD_NUMBER: _ClassVar[int]
    sandbox_id: str
    lines: int
    since_ms: int
    sources: _containers.RepeatedScalarFieldContainer[str]
    min_level: str
    def __init__(
        self,
        sandbox_id: _Optional[str] = ...,
        lines: _Optional[int] = ...,
        since_ms: _Optional[int] = ...,
        sources: _Optional[_Iterable[str]] = ...,
        min_level: _Optional[str] = ...,
    ) -> None: ...

class PushSandboxLogsRequest(_message.Message):
    __slots__ = ("sandbox_id", "logs")
    SANDBOX_ID_FIELD_NUMBER: _ClassVar[int]
    LOGS_FIELD_NUMBER: _ClassVar[int]
    sandbox_id: str
    logs: _containers.RepeatedCompositeFieldContainer[SandboxLogLine]
    def __init__(
        self,
        sandbox_id: _Optional[str] = ...,
        logs: _Optional[_Iterable[_Union[SandboxLogLine, _Mapping]]] = ...,
    ) -> None: ...

class PushSandboxLogsResponse(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class GetSandboxLogsResponse(_message.Message):
    __slots__ = ("logs", "buffer_total")
    LOGS_FIELD_NUMBER: _ClassVar[int]
    BUFFER_TOTAL_FIELD_NUMBER: _ClassVar[int]
    logs: _containers.RepeatedCompositeFieldContainer[SandboxLogLine]
    buffer_total: int
    def __init__(
        self,
        logs: _Optional[_Iterable[_Union[SandboxLogLine, _Mapping]]] = ...,
        buffer_total: _Optional[int] = ...,
    ) -> None: ...

class L7RequestSample(_message.Message):
    __slots__ = ("method", "path", "decision", "count")
    METHOD_FIELD_NUMBER: _ClassVar[int]
    PATH_FIELD_NUMBER: _ClassVar[int]
    DECISION_FIELD_NUMBER: _ClassVar[int]
    COUNT_FIELD_NUMBER: _ClassVar[int]
    method: str
    path: str
    decision: str
    count: int
    def __init__(
        self,
        method: _Optional[str] = ...,
        path: _Optional[str] = ...,
        decision: _Optional[str] = ...,
        count: _Optional[int] = ...,
    ) -> None: ...

class DenialSummary(_message.Message):
    __slots__ = (
        "sandbox_id",
        "host",
        "port",
        "binary",
        "ancestors",
        "deny_reason",
        "first_seen_ms",
        "last_seen_ms",
        "count",
        "suppressed_count",
        "total_count",
        "sample_cmdlines",
        "binary_sha256",
        "persistent",
        "denial_stage",
        "l7_request_samples",
        "l7_inspection_active",
    )
    SANDBOX_ID_FIELD_NUMBER: _ClassVar[int]
    HOST_FIELD_NUMBER: _ClassVar[int]
    PORT_FIELD_NUMBER: _ClassVar[int]
    BINARY_FIELD_NUMBER: _ClassVar[int]
    ANCESTORS_FIELD_NUMBER: _ClassVar[int]
    DENY_REASON_FIELD_NUMBER: _ClassVar[int]
    FIRST_SEEN_MS_FIELD_NUMBER: _ClassVar[int]
    LAST_SEEN_MS_FIELD_NUMBER: _ClassVar[int]
    COUNT_FIELD_NUMBER: _ClassVar[int]
    SUPPRESSED_COUNT_FIELD_NUMBER: _ClassVar[int]
    TOTAL_COUNT_FIELD_NUMBER: _ClassVar[int]
    SAMPLE_CMDLINES_FIELD_NUMBER: _ClassVar[int]
    BINARY_SHA256_FIELD_NUMBER: _ClassVar[int]
    PERSISTENT_FIELD_NUMBER: _ClassVar[int]
    DENIAL_STAGE_FIELD_NUMBER: _ClassVar[int]
    L7_REQUEST_SAMPLES_FIELD_NUMBER: _ClassVar[int]
    L7_INSPECTION_ACTIVE_FIELD_NUMBER: _ClassVar[int]
    sandbox_id: str
    host: str
    port: int
    binary: str
    ancestors: _containers.RepeatedScalarFieldContainer[str]
    deny_reason: str
    first_seen_ms: int
    last_seen_ms: int
    count: int
    suppressed_count: int
    total_count: int
    sample_cmdlines: _containers.RepeatedScalarFieldContainer[str]
    binary_sha256: str
    persistent: bool
    denial_stage: str
    l7_request_samples: _containers.RepeatedCompositeFieldContainer[L7RequestSample]
    l7_inspection_active: bool
    def __init__(
        self,
        sandbox_id: _Optional[str] = ...,
        host: _Optional[str] = ...,
        port: _Optional[int] = ...,
        binary: _Optional[str] = ...,
        ancestors: _Optional[_Iterable[str]] = ...,
        deny_reason: _Optional[str] = ...,
        first_seen_ms: _Optional[int] = ...,
        last_seen_ms: _Optional[int] = ...,
        count: _Optional[int] = ...,
        suppressed_count: _Optional[int] = ...,
        total_count: _Optional[int] = ...,
        sample_cmdlines: _Optional[_Iterable[str]] = ...,
        binary_sha256: _Optional[str] = ...,
        persistent: bool = ...,
        denial_stage: _Optional[str] = ...,
        l7_request_samples: _Optional[_Iterable[_Union[L7RequestSample, _Mapping]]] = ...,
        l7_inspection_active: bool = ...,
    ) -> None: ...

class PolicyChunk(_message.Message):
    __slots__ = (
        "id",
        "status",
        "rule_name",
        "proposed_rule",
        "rationale",
        "security_notes",
        "confidence",
        "denial_summary_ids",
        "created_at_ms",
        "decided_at_ms",
        "stage",
        "supersedes_chunk_id",
        "hit_count",
        "first_seen_ms",
        "last_seen_ms",
        "binary",
    )
    ID_FIELD_NUMBER: _ClassVar[int]
    STATUS_FIELD_NUMBER: _ClassVar[int]
    RULE_NAME_FIELD_NUMBER: _ClassVar[int]
    PROPOSED_RULE_FIELD_NUMBER: _ClassVar[int]
    RATIONALE_FIELD_NUMBER: _ClassVar[int]
    SECURITY_NOTES_FIELD_NUMBER: _ClassVar[int]
    CONFIDENCE_FIELD_NUMBER: _ClassVar[int]
    DENIAL_SUMMARY_IDS_FIELD_NUMBER: _ClassVar[int]
    CREATED_AT_MS_FIELD_NUMBER: _ClassVar[int]
    DECIDED_AT_MS_FIELD_NUMBER: _ClassVar[int]
    STAGE_FIELD_NUMBER: _ClassVar[int]
    SUPERSEDES_CHUNK_ID_FIELD_NUMBER: _ClassVar[int]
    HIT_COUNT_FIELD_NUMBER: _ClassVar[int]
    FIRST_SEEN_MS_FIELD_NUMBER: _ClassVar[int]
    LAST_SEEN_MS_FIELD_NUMBER: _ClassVar[int]
    BINARY_FIELD_NUMBER: _ClassVar[int]
    id: str
    status: str
    rule_name: str
    proposed_rule: _sandbox_pb2.NetworkPolicyRule
    rationale: str
    security_notes: str
    confidence: float
    denial_summary_ids: _containers.RepeatedScalarFieldContainer[str]
    created_at_ms: int
    decided_at_ms: int
    stage: str
    supersedes_chunk_id: str
    hit_count: int
    first_seen_ms: int
    last_seen_ms: int
    binary: str
    def __init__(
        self,
        id: _Optional[str] = ...,
        status: _Optional[str] = ...,
        rule_name: _Optional[str] = ...,
        proposed_rule: _Optional[_Union[_sandbox_pb2.NetworkPolicyRule, _Mapping]] = ...,
        rationale: _Optional[str] = ...,
        security_notes: _Optional[str] = ...,
        confidence: _Optional[float] = ...,
        denial_summary_ids: _Optional[_Iterable[str]] = ...,
        created_at_ms: _Optional[int] = ...,
        decided_at_ms: _Optional[int] = ...,
        stage: _Optional[str] = ...,
        supersedes_chunk_id: _Optional[str] = ...,
        hit_count: _Optional[int] = ...,
        first_seen_ms: _Optional[int] = ...,
        last_seen_ms: _Optional[int] = ...,
        binary: _Optional[str] = ...,
    ) -> None: ...

class DraftPolicyUpdate(_message.Message):
    __slots__ = ("draft_version", "new_chunks", "total_pending", "summary")
    DRAFT_VERSION_FIELD_NUMBER: _ClassVar[int]
    NEW_CHUNKS_FIELD_NUMBER: _ClassVar[int]
    TOTAL_PENDING_FIELD_NUMBER: _ClassVar[int]
    SUMMARY_FIELD_NUMBER: _ClassVar[int]
    draft_version: int
    new_chunks: int
    total_pending: int
    summary: str
    def __init__(
        self,
        draft_version: _Optional[int] = ...,
        new_chunks: _Optional[int] = ...,
        total_pending: _Optional[int] = ...,
        summary: _Optional[str] = ...,
    ) -> None: ...

class SubmitPolicyAnalysisRequest(_message.Message):
    __slots__ = ("summaries", "proposed_chunks", "analysis_mode", "name")
    SUMMARIES_FIELD_NUMBER: _ClassVar[int]
    PROPOSED_CHUNKS_FIELD_NUMBER: _ClassVar[int]
    ANALYSIS_MODE_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    summaries: _containers.RepeatedCompositeFieldContainer[DenialSummary]
    proposed_chunks: _containers.RepeatedCompositeFieldContainer[PolicyChunk]
    analysis_mode: str
    name: str
    def __init__(
        self,
        summaries: _Optional[_Iterable[_Union[DenialSummary, _Mapping]]] = ...,
        proposed_chunks: _Optional[_Iterable[_Union[PolicyChunk, _Mapping]]] = ...,
        analysis_mode: _Optional[str] = ...,
        name: _Optional[str] = ...,
    ) -> None: ...

class SubmitPolicyAnalysisResponse(_message.Message):
    __slots__ = ("accepted_chunks", "rejected_chunks", "rejection_reasons")
    ACCEPTED_CHUNKS_FIELD_NUMBER: _ClassVar[int]
    REJECTED_CHUNKS_FIELD_NUMBER: _ClassVar[int]
    REJECTION_REASONS_FIELD_NUMBER: _ClassVar[int]
    accepted_chunks: int
    rejected_chunks: int
    rejection_reasons: _containers.RepeatedScalarFieldContainer[str]
    def __init__(
        self,
        accepted_chunks: _Optional[int] = ...,
        rejected_chunks: _Optional[int] = ...,
        rejection_reasons: _Optional[_Iterable[str]] = ...,
    ) -> None: ...

class GetDraftPolicyRequest(_message.Message):
    __slots__ = ("name", "status_filter")
    NAME_FIELD_NUMBER: _ClassVar[int]
    STATUS_FILTER_FIELD_NUMBER: _ClassVar[int]
    name: str
    status_filter: str
    def __init__(self, name: _Optional[str] = ..., status_filter: _Optional[str] = ...) -> None: ...

class GetDraftPolicyResponse(_message.Message):
    __slots__ = ("chunks", "rolling_summary", "draft_version", "last_analyzed_at_ms")
    CHUNKS_FIELD_NUMBER: _ClassVar[int]
    ROLLING_SUMMARY_FIELD_NUMBER: _ClassVar[int]
    DRAFT_VERSION_FIELD_NUMBER: _ClassVar[int]
    LAST_ANALYZED_AT_MS_FIELD_NUMBER: _ClassVar[int]
    chunks: _containers.RepeatedCompositeFieldContainer[PolicyChunk]
    rolling_summary: str
    draft_version: int
    last_analyzed_at_ms: int
    def __init__(
        self,
        chunks: _Optional[_Iterable[_Union[PolicyChunk, _Mapping]]] = ...,
        rolling_summary: _Optional[str] = ...,
        draft_version: _Optional[int] = ...,
        last_analyzed_at_ms: _Optional[int] = ...,
    ) -> None: ...

class ApproveDraftChunkRequest(_message.Message):
    __slots__ = ("name", "chunk_id")
    NAME_FIELD_NUMBER: _ClassVar[int]
    CHUNK_ID_FIELD_NUMBER: _ClassVar[int]
    name: str
    chunk_id: str
    def __init__(self, name: _Optional[str] = ..., chunk_id: _Optional[str] = ...) -> None: ...

class ApproveDraftChunkResponse(_message.Message):
    __slots__ = ("policy_version", "policy_hash")
    POLICY_VERSION_FIELD_NUMBER: _ClassVar[int]
    POLICY_HASH_FIELD_NUMBER: _ClassVar[int]
    policy_version: int
    policy_hash: str
    def __init__(
        self, policy_version: _Optional[int] = ..., policy_hash: _Optional[str] = ...
    ) -> None: ...

class RejectDraftChunkRequest(_message.Message):
    __slots__ = ("name", "chunk_id", "reason")
    NAME_FIELD_NUMBER: _ClassVar[int]
    CHUNK_ID_FIELD_NUMBER: _ClassVar[int]
    REASON_FIELD_NUMBER: _ClassVar[int]
    name: str
    chunk_id: str
    reason: str
    def __init__(
        self,
        name: _Optional[str] = ...,
        chunk_id: _Optional[str] = ...,
        reason: _Optional[str] = ...,
    ) -> None: ...

class RejectDraftChunkResponse(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class ApproveAllDraftChunksRequest(_message.Message):
    __slots__ = ("name", "include_security_flagged")
    NAME_FIELD_NUMBER: _ClassVar[int]
    INCLUDE_SECURITY_FLAGGED_FIELD_NUMBER: _ClassVar[int]
    name: str
    include_security_flagged: bool
    def __init__(
        self, name: _Optional[str] = ..., include_security_flagged: bool = ...
    ) -> None: ...

class ApproveAllDraftChunksResponse(_message.Message):
    __slots__ = ("policy_version", "policy_hash", "chunks_approved", "chunks_skipped")
    POLICY_VERSION_FIELD_NUMBER: _ClassVar[int]
    POLICY_HASH_FIELD_NUMBER: _ClassVar[int]
    CHUNKS_APPROVED_FIELD_NUMBER: _ClassVar[int]
    CHUNKS_SKIPPED_FIELD_NUMBER: _ClassVar[int]
    policy_version: int
    policy_hash: str
    chunks_approved: int
    chunks_skipped: int
    def __init__(
        self,
        policy_version: _Optional[int] = ...,
        policy_hash: _Optional[str] = ...,
        chunks_approved: _Optional[int] = ...,
        chunks_skipped: _Optional[int] = ...,
    ) -> None: ...

class EditDraftChunkRequest(_message.Message):
    __slots__ = ("name", "chunk_id", "proposed_rule")
    NAME_FIELD_NUMBER: _ClassVar[int]
    CHUNK_ID_FIELD_NUMBER: _ClassVar[int]
    PROPOSED_RULE_FIELD_NUMBER: _ClassVar[int]
    name: str
    chunk_id: str
    proposed_rule: _sandbox_pb2.NetworkPolicyRule
    def __init__(
        self,
        name: _Optional[str] = ...,
        chunk_id: _Optional[str] = ...,
        proposed_rule: _Optional[_Union[_sandbox_pb2.NetworkPolicyRule, _Mapping]] = ...,
    ) -> None: ...

class EditDraftChunkResponse(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class UndoDraftChunkRequest(_message.Message):
    __slots__ = ("name", "chunk_id")
    NAME_FIELD_NUMBER: _ClassVar[int]
    CHUNK_ID_FIELD_NUMBER: _ClassVar[int]
    name: str
    chunk_id: str
    def __init__(self, name: _Optional[str] = ..., chunk_id: _Optional[str] = ...) -> None: ...

class UndoDraftChunkResponse(_message.Message):
    __slots__ = ("policy_version", "policy_hash")
    POLICY_VERSION_FIELD_NUMBER: _ClassVar[int]
    POLICY_HASH_FIELD_NUMBER: _ClassVar[int]
    policy_version: int
    policy_hash: str
    def __init__(
        self, policy_version: _Optional[int] = ..., policy_hash: _Optional[str] = ...
    ) -> None: ...

class ClearDraftChunksRequest(_message.Message):
    __slots__ = ("name",)
    NAME_FIELD_NUMBER: _ClassVar[int]
    name: str
    def __init__(self, name: _Optional[str] = ...) -> None: ...

class ClearDraftChunksResponse(_message.Message):
    __slots__ = ("chunks_cleared",)
    CHUNKS_CLEARED_FIELD_NUMBER: _ClassVar[int]
    chunks_cleared: int
    def __init__(self, chunks_cleared: _Optional[int] = ...) -> None: ...

class GetDraftHistoryRequest(_message.Message):
    __slots__ = ("name",)
    NAME_FIELD_NUMBER: _ClassVar[int]
    name: str
    def __init__(self, name: _Optional[str] = ...) -> None: ...

class DraftHistoryEntry(_message.Message):
    __slots__ = ("timestamp_ms", "event_type", "description", "chunk_id")
    TIMESTAMP_MS_FIELD_NUMBER: _ClassVar[int]
    EVENT_TYPE_FIELD_NUMBER: _ClassVar[int]
    DESCRIPTION_FIELD_NUMBER: _ClassVar[int]
    CHUNK_ID_FIELD_NUMBER: _ClassVar[int]
    timestamp_ms: int
    event_type: str
    description: str
    chunk_id: str
    def __init__(
        self,
        timestamp_ms: _Optional[int] = ...,
        event_type: _Optional[str] = ...,
        description: _Optional[str] = ...,
        chunk_id: _Optional[str] = ...,
    ) -> None: ...

class GetDraftHistoryResponse(_message.Message):
    __slots__ = ("entries",)
    ENTRIES_FIELD_NUMBER: _ClassVar[int]
    entries: _containers.RepeatedCompositeFieldContainer[DraftHistoryEntry]
    def __init__(
        self, entries: _Optional[_Iterable[_Union[DraftHistoryEntry, _Mapping]]] = ...
    ) -> None: ...



"""SQLAlchemy ORM models for Shoreguard."""

from __future__ import annotations

import datetime

from sqlalchemy import (
    Boolean,
    DateTime,
    ForeignKey,
    Integer,
    LargeBinary,
    String,
    Text,
    UniqueConstraint,
)
from sqlalchemy.orm import DeclarativeBase, Mapped, mapped_column


class Base(DeclarativeBase):
    """Base class for all Shoreguard models."""


class Gateway(Base):
    """A registered OpenShell gateway.

    Attributes:
        id: Auto-incremented primary key.
        name: Unique gateway name (max 253 chars).
        endpoint: Host:port of the gateway gRPC endpoint.
        scheme: Connection scheme (``https`` by default).
        auth_mode: Optional authentication mode identifier.
        ca_cert: Optional CA certificate bytes for mTLS.
        client_cert: Optional client certificate bytes for mTLS.
        client_key: Optional client private key bytes for mTLS.
        metadata_json: Optional JSON-encoded metadata blob.
        description: Optional free-text description of the gateway's purpose.
        labels_json: Optional JSON-encoded key-value labels for filtering.
        registered_at: Timestamp when the gateway was registered.
        last_seen: Timestamp of the most recent health check.
        last_status: Last known health status string.
    """

    __tablename__ = "gateways"

    id: Mapped[int] = mapped_column(Integer, primary_key=True, autoincrement=True)
    name: Mapped[str] = mapped_column(String(253), unique=True, nullable=False)
    endpoint: Mapped[str] = mapped_column(String(260), nullable=False)
    scheme: Mapped[str] = mapped_column(String, nullable=False, default="https")
    auth_mode: Mapped[str | None] = mapped_column(String)
    ca_cert: Mapped[bytes | None] = mapped_column(LargeBinary)
    client_cert: Mapped[bytes | None] = mapped_column(LargeBinary)
    client_key: Mapped[bytes | None] = mapped_column(LargeBinary)
    metadata_json: Mapped[str | None] = mapped_column(Text)
    description: Mapped[str | None] = mapped_column(Text)
    labels_json: Mapped[str | None] = mapped_column(Text)
    registered_at: Mapped[datetime.datetime] = mapped_column(
        DateTime(timezone=True),
        nullable=False,
    )
    last_seen: Mapped[datetime.datetime | None] = mapped_column(DateTime(timezone=True))
    last_status: Mapped[str] = mapped_column(String, default="unknown")


class User(Base):
    """A user account with email/password authentication.

    Attributes:
        id: Auto-incremented primary key.
        email: Unique email address (max 254 chars).
        hashed_password: Bcrypt-hashed password, or ``None`` for invite-only.
        role: Global role (``admin``, ``operator``, ``viewer``).
        is_active: Whether the account is enabled.
        invite_token_hash: SHA-256 hash of the invite token, if pending.
        created_at: Timestamp when the user was created.
    """

    __tablename__ = "users"

    id: Mapped[int] = mapped_column(Integer, primary_key=True, autoincrement=True)
    email: Mapped[str] = mapped_column(String(254), unique=True, nullable=False)
    hashed_password: Mapped[str | None] = mapped_column(String(128))
    role: Mapped[str] = mapped_column(String(20), nullable=False, default="viewer")
    is_active: Mapped[bool] = mapped_column(Boolean, nullable=False, default=True)
    invite_token_hash: Mapped[str | None] = mapped_column(String(64))
    created_at: Mapped[datetime.datetime] = mapped_column(DateTime(timezone=True), nullable=False)


class ServicePrincipal(Base):
    """A service principal (API key) for programmatic access.

    Attributes:
        id: Auto-incremented primary key.
        name: Unique human-readable name (max 100 chars).
        key_hash: SHA-256 hash of the API key.
        key_prefix: First 12 characters of the key for identification.
        role: Global role (``admin``, ``operator``, ``viewer``).
        created_by: FK to the user who created this principal, or ``None``.
        created_at: Timestamp when the principal was created.
        last_used: Timestamp of the most recent API call, or ``None``.
        expires_at: Optional expiry timestamp; ``None`` means never expires.
    """

    __tablename__ = "service_principals"

    id: Mapped[int] = mapped_column(Integer, primary_key=True, autoincrement=True)
    name: Mapped[str] = mapped_column(String(100), unique=True, nullable=False)
    key_hash: Mapped[str] = mapped_column(String(64), nullable=False)
    key_prefix: Mapped[str | None] = mapped_column(String(12))
    role: Mapped[str] = mapped_column(String(20), nullable=False, default="viewer")
    created_by: Mapped[int | None] = mapped_column(
        Integer, ForeignKey("users.id", ondelete="SET NULL")
    )
    created_at: Mapped[datetime.datetime] = mapped_column(DateTime(timezone=True), nullable=False)
    last_used: Mapped[datetime.datetime | None] = mapped_column(DateTime(timezone=True))
    expires_at: Mapped[datetime.datetime | None] = mapped_column(DateTime(timezone=True))


class UserGatewayRole(Base):
    """A per-gateway role override for a user.

    Attributes:
        id: Auto-incremented primary key.
        user_id: FK to the user.
        gateway_id: FK to the gateway.
        role: Scoped role for this user on this gateway.
    """

    __tablename__ = "user_gateway_roles"
    __table_args__ = (UniqueConstraint("user_id", "gateway_id"),)

    id: Mapped[int] = mapped_column(Integer, primary_key=True, autoincrement=True)
    user_id: Mapped[int] = mapped_column(
        Integer, ForeignKey("users.id", ondelete="CASCADE"), nullable=False
    )
    gateway_id: Mapped[int] = mapped_column(
        Integer, ForeignKey("gateways.id", ondelete="CASCADE"), nullable=False
    )
    role: Mapped[str] = mapped_column(String(20), nullable=False)


class SPGatewayRole(Base):
    """A per-gateway role override for a service principal.

    Attributes:
        id: Auto-incremented primary key.
        sp_id: FK to the service principal.
        gateway_id: FK to the gateway.
        role: Scoped role for this principal on this gateway.
    """

    __tablename__ = "sp_gateway_roles"
    __table_args__ = (UniqueConstraint("sp_id", "gateway_id"),)

    id: Mapped[int] = mapped_column(Integer, primary_key=True, autoincrement=True)
    sp_id: Mapped[int] = mapped_column(
        Integer, ForeignKey("service_principals.id", ondelete="CASCADE"), nullable=False
    )
    gateway_id: Mapped[int] = mapped_column(
        Integer, ForeignKey("gateways.id", ondelete="CASCADE"), nullable=False
    )
    role: Mapped[str] = mapped_column(String(20), nullable=False)


class AuditEntry(Base):
    """A persistent audit log entry for state-changing operations.

    Attributes:
        id: Auto-incremented primary key.
        timestamp: When the action occurred.
        actor: Email or service principal name of the acting identity.
        actor_role: Effective role at time of action.
        action: Machine-readable action identifier.
        resource_type: Type of resource affected (e.g. ``sandbox``).
        resource_id: Identifier of the affected resource.
        gateway_name: Human-readable gateway name, if applicable.
        gateway_id: FK to the gateway, or ``None`` if deleted.
        detail: Optional free-text detail or JSON payload.
        client_ip: IP address of the requesting client, if available.
    """

    __tablename__ = "audit_log"

    id: Mapped[int] = mapped_column(Integer, primary_key=True, autoincrement=True)
    timestamp: Mapped[datetime.datetime] = mapped_column(DateTime(timezone=True), nullable=False)
    actor: Mapped[str] = mapped_column(String(254), nullable=False)
    actor_role: Mapped[str] = mapped_column(
        String(20), nullable=False, default="unknown", server_default="unknown"
    )
    action: Mapped[str] = mapped_column(String(100), nullable=False)
    resource_type: Mapped[str] = mapped_column(String(50), nullable=False)
    resource_id: Mapped[str] = mapped_column(
        String(253), nullable=False, default="", server_default=""
    )
    gateway_name: Mapped[str | None] = mapped_column(String(253))
    gateway_id: Mapped[int | None] = mapped_column(
        Integer, ForeignKey("gateways.id", ondelete="SET NULL")
    )
    detail: Mapped[str | None] = mapped_column(Text)
    client_ip: Mapped[str | None] = mapped_column(String(45))


class Webhook(Base):
    """A webhook endpoint for event notifications.

    Attributes:
        id: Auto-incremented primary key.
        url: Target URL for POST requests (max 2048 chars).
        secret: HMAC-SHA256 signing secret.
        event_types: JSON-encoded list of subscribed event types.
        is_active: Whether the webhook is enabled.
        channel_type: Channel type (generic, slack, discord, email).
        extra_config: Optional JSON config for channel-specific settings.
        created_by: Email or service principal name of the creator.
        created_at: Timestamp when the webhook was created.
    """

    __tablename__ = "webhooks"

    id: Mapped[int] = mapped_column(Integer, primary_key=True, autoincrement=True)
    url: Mapped[str] = mapped_column(String(2048), nullable=False)
    secret: Mapped[str] = mapped_column(String(128), nullable=False)
    event_types: Mapped[str] = mapped_column(
        Text, nullable=False, default="[]", server_default="[]"
    )
    is_active: Mapped[bool] = mapped_column(Boolean, nullable=False, default=True)
    channel_type: Mapped[str] = mapped_column(
        String(20), nullable=False, default="generic", server_default="generic"
    )
    extra_config: Mapped[str | None] = mapped_column(Text)
    created_by: Mapped[str] = mapped_column(String(254), nullable=False)
    created_at: Mapped[datetime.datetime] = mapped_column(DateTime(timezone=True), nullable=False)


class WebhookDelivery(Base):
    """A delivery attempt for a webhook event.

    Attributes:
        id: Auto-incremented primary key.
        webhook_id: FK to the webhook that was triggered.
        event_type: The event type that triggered the delivery.
        payload_json: JSON-encoded event payload.
        status: Delivery status (``pending``, ``success``, ``failed``).
        response_code: HTTP response code from the target, if any.
        error_message: Error details on failure, if any.
        attempt: Current attempt number (1-based).
        created_at: Timestamp when the delivery was created.
        delivered_at: Timestamp when delivery succeeded, if any.
    """

    __tablename__ = "webhook_deliveries"

    id: Mapped[int] = mapped_column(Integer, primary_key=True, autoincrement=True)
    webhook_id: Mapped[int] = mapped_column(
        Integer, ForeignKey("webhooks.id", ondelete="CASCADE"), nullable=False
    )
    event_type: Mapped[str] = mapped_column(String(100), nullable=False)
    payload_json: Mapped[str] = mapped_column(Text, nullable=False)
    status: Mapped[str] = mapped_column(
        String(20), nullable=False, default="pending", server_default="pending"
    )
    response_code: Mapped[int | None] = mapped_column(Integer)
    error_message: Mapped[str | None] = mapped_column(Text)
    attempt: Mapped[int] = mapped_column(Integer, nullable=False, default=1)
    created_at: Mapped[datetime.datetime] = mapped_column(DateTime(timezone=True), nullable=False)
    delivered_at: Mapped[datetime.datetime | None] = mapped_column(DateTime(timezone=True))



"""Gateway connection management and registry-backed discovery."""

from __future__ import annotations

import logging
import os
import threading
import time
from typing import Any

import grpc

from shoreguard.client import ShoreGuardClient
from shoreguard.config import is_private_ip
from shoreguard.exceptions import GatewayNotConnectedError, NotFoundError
from shoreguard.services.registry import _UNSET, GatewayRegistry

logger = logging.getLogger(__name__)

# ─── Connection state ────────────────────────────────────────────────────────

_BACKOFF_MIN = 5.0
_BACKOFF_MAX = 60.0
_BACKOFF_FACTOR = 2.0


class _ClientEntry:
    """Per-gateway connection state with backoff."""

    __slots__ = ("client", "last_attempt", "backoff")

    def __init__(self) -> None:  # noqa: D107
        self.client: ShoreGuardClient | None = None
        self.last_attempt: float = 0.0
        self.backoff: float = 0.0


_clients: dict[str, _ClientEntry] = {}
_clients_lock = threading.Lock()


def _reset_clients() -> None:
    """Clear all cached gateway clients. For test teardown only."""
    with _clients_lock:
        _clients.clear()


def _derive_status(connected: bool, last_status: str | None) -> str:
    """Derive a single status string from connection and health state.

    Args:
        connected: Whether the gateway is currently connected.
        last_status: Last known health status from the registry.

    Returns:
        str: Derived status string.
    """
    if connected:
        return "connected"
    if last_status in ("healthy", "degraded"):
        return "unreachable"
    return "offline"


# ─── Gateway Service ─────────────────────────────────────────────────────────


class GatewayService:
    """Gateway connection management and registry-backed discovery.

    Handles gRPC client connections, backoff, health probing,
    and gateway registration/unregistration.

    Args:
        registry: Gateway registry for persistence.

    Attributes:
        registry: The underlying gateway registry.
    """

    def __init__(self, registry: GatewayRegistry) -> None:  # noqa: D107
        self._registry = registry

    @property
    def registry(self) -> GatewayRegistry:
        """The underlying gateway registry."""
        return self._registry

    # ── Connection management ─────────────────────────────────────────────

    def get_client(self, name: str) -> ShoreGuardClient:
        """Return a client for the given gateway, attempting reconnect with backoff.

        Args:
            name: Gateway name.

        Returns:
            ShoreGuardClient: Connected gRPC client.

        Raises:
            GatewayNotConnectedError: If connection fails.
        """
        gw_name = name

        # Phase 1: read state under the lock
        with _clients_lock:
            entry = _clients.get(gw_name)
            if entry is None:
                entry = _ClientEntry()
                _clients[gw_name] = entry
            existing_client = entry.client

        # Phase 2: health-check existing client (blocking I/O, no lock)
        if existing_client is not None:
            try:
                existing_client.health()
                return existing_client
            except grpc.RpcError:
                logger.warning("Gateway '%s' connection lost, attempting reconnect...", gw_name)
                try:
                    existing_client.close()
                except Exception:
                    logger.debug("Error closing stale connection for '%s'", gw_name, exc_info=True)
                with _clients_lock:
                    entry.client = None
                    entry.backoff = 0.0

        # Phase 3: check backoff under the lock
        now = time.monotonic()
        with _clients_lock:
            if entry.backoff > 0 and (now - entry.last_attempt) < entry.backoff:
                raise GatewayNotConnectedError(f"Gateway '{gw_name}' not connected.")
            entry.last_attempt = now

        # Phase 4: attempt connection (blocking I/O, no lock)
        new_client = self._try_connect(gw_name)

        # Phase 5: write result under the lock
        with _clients_lock:
            if new_client is None:
                if entry.backoff == 0:
                    entry.backoff = _BACKOFF_MIN
                else:
                    entry.backoff = min(entry.backoff * _BACKOFF_FACTOR, _BACKOFF_MAX)
                raise GatewayNotConnectedError(f"Gateway '{gw_name}' not connected.")
            entry.client = new_client
            entry.backoff = 0.0
        return new_client

    def set_client(self, client: ShoreGuardClient | None, name: str) -> None:
        """Set or clear a client for the given gateway.

        Args:
            client: Client to cache, or None to clear.
            name: Gateway name.
        """
        gw_name = name
        with _clients_lock:
            if client is None:
                _clients.pop(gw_name, None)
                logger.debug("Cleared client for gateway '%s'", gw_name)
            else:
                entry = _clients.get(gw_name)
                if entry is None:
                    entry = _ClientEntry()
                    _clients[gw_name] = entry
                entry.client = client
                entry.backoff = 0.0
                logger.debug("Set client for gateway '%s'", gw_name)

    def reset_backoff(self, name: str) -> None:
        """Reset connection backoff for a gateway.

        Args:
            name: Gateway name.
        """
        gw_name = name
        with _clients_lock:
            if gw_name and gw_name in _clients:
                _clients[gw_name].backoff = 0.0
                _clients[gw_name].last_attempt = 0.0
                logger.debug("Reset backoff for gateway '%s'", gw_name)

    def _try_connect(self, name: str) -> ShoreGuardClient | None:
        """Attempt to create a client for a specific gateway.

        Args:
            name: Gateway name.

        Returns:
            ShoreGuardClient | None: Connected client, or None on failure.
        """
        creds = self._registry.get_credentials(name)
        if creds is not None:
            return self._try_connect_from_registry(name, creds)
        return self._try_connect_from_config(name)

    def _try_connect_from_registry(
        self, name: str, creds: dict[str, str | bytes | None]
    ) -> ShoreGuardClient | None:
        """Connect using credentials from the database.

        Args:
            name: Gateway name.
            creds: Credential dict from the registry.

        Returns:
            ShoreGuardClient | None: Connected client, or None on failure.
        """
        endpoint = str(creds["endpoint"])
        host = endpoint.rsplit(":", 1)[0] if ":" in endpoint else endpoint
        if is_private_ip(host) and not os.environ.get("SHOREGUARD_LOCAL_MODE"):
            logger.warning(
                "Gateway '%s' endpoint '%s' resolves to a private IP — blocking connection",
                name,
                endpoint,
            )
            return None
        ca_cert = creds.get("ca_cert")
        client_cert = creds.get("client_cert")
        client_key = creds.get("client_key")
        try:
            client = ShoreGuardClient.from_credentials(
                endpoint,
                ca_cert=ca_cert if isinstance(ca_cert, bytes) else None,
                client_cert=client_cert if isinstance(client_cert, bytes) else None,
                client_key=client_key if isinstance(client_key, bytes) else None,
            )
        except (grpc.RpcError, OSError, ConnectionError, TimeoutError) as e:
            logger.debug("Gateway '%s' connection failed (type=%s): %s", name, type(e).__name__, e)
            return None
        try:
            client.health()
            logger.info("Connected to OpenShell gateway '%s'", name)
            return client
        except (grpc.RpcError, OSError, ConnectionError, TimeoutError) as e:
            logger.debug(
                "Gateway '%s' health check failed (type=%s): %s",
                name,
                type(e).__name__,
                e,
            )
            try:
                client.close()
            except (grpc.RpcError, OSError):
                logger.debug("Failed to close client for '%s'", name)
            return None

    def _try_connect_from_config(self, name: str) -> ShoreGuardClient | None:
        """Fallback: connect using filesystem config (local mode / backward compat).

        Args:
            name: Gateway name.

        Returns:
            ShoreGuardClient | None: Connected client, or None on failure.
        """
        import json

        try:
            client = ShoreGuardClient.from_active_cluster(cluster=name)
        except (
            grpc.RpcError,
            GatewayNotConnectedError,
            OSError,
            ConnectionError,
            TimeoutError,
            KeyError,
            ValueError,
            json.JSONDecodeError,
        ) as e:
            logger.debug("Gateway '%s' connection failed: %s", name, e, exc_info=True)
            return None
        try:
            client.health()
            logger.info("Connected to OpenShell gateway '%s'", name)
            return client
        except (grpc.RpcError, OSError, ConnectionError, TimeoutError) as e:
            logger.debug("Gateway '%s' health check failed: %s", name, e, exc_info=True)
            try:
                client.close()
            except (grpc.RpcError, OSError):
                logger.debug("Failed to close client for '%s'", name)
            return None

    # ── Registration ─────────────────────────────────────────────────────

    def register(
        self,
        name: str,
        endpoint: str,
        scheme: str = "https",
        auth_mode: str | None = "mtls",
        *,
        ca_cert: bytes | None = None,
        client_cert: bytes | None = None,
        client_key: bytes | None = None,
        metadata: dict[str, Any] | None = None,
        description: str | None = None,
        labels: dict[str, str] | None = None,
    ) -> dict[str, Any]:
        """Register a gateway and attempt initial connection.

        Args:
            name: Unique gateway name.
            endpoint: Gateway endpoint address.
            scheme: Connection scheme (e.g. "https").
            auth_mode: Authentication mode (e.g. "mtls").
            ca_cert: CA certificate bytes for TLS.
            client_cert: Client certificate bytes for mTLS.
            client_key: Client private key bytes for mTLS.
            metadata: Optional metadata dict.
            description: Optional free-text description.
            labels: Optional key-value labels for filtering.

        Returns:
            dict[str, Any]: Gateway record with connection status.
        """
        logger.info("Registering gateway '%s' (endpoint=%s)", name, endpoint)
        record = self._registry.register(
            name,
            endpoint,
            scheme,
            auth_mode,
            ca_cert=ca_cert,
            client_cert=client_cert,
            client_key=client_key,
            metadata=metadata,
            description=description,
            labels=labels,
        )

        # Attempt connection to validate
        connected = False
        try:
            self.get_client(name=name)
            connected = True
        except (GatewayNotConnectedError, grpc.RpcError):
            logger.debug("Could not connect to newly registered gateway '%s'", name)

        record["connected"] = connected
        record["status"] = "connected" if connected else "unreachable"

        return record

    def unregister(self, name: str) -> bool:
        """Unregister a gateway and close its connection.

        Args:
            name: Gateway name.

        Returns:
            bool: True if the gateway existed and was removed.
        """
        logger.info("Unregistering gateway '%s'", name)
        self.set_client(None, name=name)
        return self._registry.unregister(name)

    def test_connection(self, name: str) -> dict[str, Any]:
        """Explicitly test connectivity to a registered gateway.

        Args:
            name: Gateway name.

        Returns:
            dict[str, Any]: Connection test result.

        Raises:
            NotFoundError: If the gateway is not registered.
        """
        record = self._registry.get(name)
        if record is None:
            raise NotFoundError(f"Gateway '{name}' not registered")

        self.reset_backoff(name)
        try:
            client = self.get_client(name=name)
            health = client.health()
            return {
                "success": True,
                "connected": True,
                "version": health.get("version"),
                "health_status": health.get("status"),
            }
        except (GatewayNotConnectedError, grpc.RpcError) as e:
            return {"success": False, "connected": False, "error": str(e)}

    # ── List & Info ───────────────────────────────────────────────────────

    def update_gateway_metadata(
        self,
        name: str,
        *,
        description: str | None | object = _UNSET,
        labels: dict[str, str] | None | object = _UNSET,
    ) -> dict[str, Any]:
        """Update description and/or labels for a gateway.

        Args:
            name: Gateway name.
            description: New description, None to clear, or sentinel to skip.
            labels: New labels dict, None to clear, or sentinel to skip.

        Returns:
            dict[str, Any]: Updated gateway record.

        Raises:
            NotFoundError: If the gateway does not exist.
        """
        kwargs: dict[str, Any] = {}
        if description is not _UNSET:
            kwargs["description"] = description
        if labels is not _UNSET:
            kwargs["labels"] = labels
        result = self._registry.update_gateway_metadata(name, **kwargs)
        if result is None:
            raise NotFoundError(f"Gateway '{name}' not found")
        return result

    def list_all(self, *, labels_filter: dict[str, str] | None = None) -> list[dict[str, Any]]:
        """List all registered gateways with cached connection status.

        Uses the cached client state instead of live health probes to avoid
        N+1 blocking gRPC calls.  The background health monitor keeps
        ``last_status`` up-to-date.

        Args:
            labels_filter: If provided, only return gateways matching all
                specified label key-value pairs.

        Returns:
            list[dict[str, Any]]: Gateway records with connection status.
        """
        gateways = self._registry.list_all(labels_filter=labels_filter)

        for gw in gateways:
            with _clients_lock:
                cached = _clients.get(gw["name"])
                connected = cached is not None and cached.client is not None

            gw["connected"] = connected
            gw["status"] = _derive_status(connected, gw.get("last_status"))

        return gateways

    def get_info(self, name: str) -> dict[str, Any]:
        """Get detailed info for a gateway.

        Args:
            name: Gateway name.

        Returns:
            dict[str, Any]: Detailed gateway information.
        """
        record = self._registry.get(name)
        if record is None:
            return {"configured": False, "error": f"Gateway '{name}' not registered"}

        record["configured"] = True

        connected = False
        version = None
        with _clients_lock:
            cached = _clients.get(name)
            cached_client = cached.client if cached else None
        if cached_client is not None:
            try:
                health = cached_client.health()
                connected = True
                version = health.get("version")
            except grpc.RpcError:
                self.set_client(None, name=name)

        record["connected"] = connected
        if version:
            record["version"] = version
        record["status"] = _derive_status(connected, record.get("last_status"))
        return record

    def get_config(self, name: str) -> dict[str, Any]:
        """Fetch the gateway configuration via gRPC.

        Args:
            name: Gateway name.

        Returns:
            dict[str, Any]: Gateway configuration.
        """
        client = self.get_client(name=name)
        return client.get_gateway_config()

    # ── Health monitor ────────────────────────────────────────────────────

    def check_all_health(self) -> None:
        """Probe all registered gateways and update their health in the registry."""
        from datetime import UTC, datetime

        gateways = self._registry.list_all()
        if not gateways:
            return
        logger.debug("Starting health check for %d gateway(s)", len(gateways))
        for gw in gateways:
            name = gw["name"]
            try:
                client = self.get_client(name=name)
                health = client.health()
                status = health.get("status", "unknown")
            except (GatewayNotConnectedError, grpc.RpcError) as e:
                logger.debug("Health probe failed for '%s': %s", name, e)
                status = "unreachable"
            try:
                self._registry.update_health(name, status, datetime.now(UTC))
            except Exception:
                logger.warning("Failed to update health for '%s'", name, exc_info=True)

    def get_cached_client(self, name: str) -> ShoreGuardClient | None:
        """Return the cached client for a gateway, or None if not connected.

        Args:
            name: Gateway name.

        Returns:
            ShoreGuardClient | None: Cached client, or None.
        """
        with _clients_lock:
            entry = _clients.get(name)
            if entry is not None and entry.client is not None:
                return entry.client
        return None


# Module-level reference — set during app lifespan (see shoreguard.api.main).
gateway_service: GatewayService | None = None



"""Provider management operations."""

from __future__ import annotations

from typing import Any

from ._proto import datamodel_pb2, openshell_pb2, openshell_pb2_grpc


def _provider_to_dict(provider: datamodel_pb2.Provider) -> dict[str, Any]:
    """Convert a Provider protobuf to a plain dict.

    Args:
        provider: Provider protobuf message.

    Returns:
        dict[str, Any]: Provider data with id, name, type, credentials,
            and config.
    """
    return {
        "id": provider.id,
        "name": provider.name,
        "type": provider.type,
        "credentials": dict(provider.credentials),
        "config": dict(provider.config),
    }


class ProviderManager:
    """Provider CRUD operations against OpenShell gateway.

    Args:
        stub: OpenShell gRPC stub.
        timeout: gRPC call timeout in seconds.
    """

    def __init__(self, stub: openshell_pb2_grpc.OpenShellStub, *, timeout: float = 30.0) -> None:  # noqa: D107
        self._stub = stub
        self._timeout = timeout

    def list(self, *, limit: int = 100, offset: int = 0) -> list[dict[str, Any]]:
        """List all providers.

        Args:
            limit: Maximum number of providers to return.
            offset: Pagination offset.

        Returns:
            list[dict[str, Any]]: List of provider dicts.
        """
        resp = self._stub.ListProviders(
            openshell_pb2.ListProvidersRequest(limit=limit, offset=offset),
            timeout=self._timeout,
        )
        return [_provider_to_dict(p) for p in resp.providers]

    def get(self, name: str) -> dict[str, Any]:
        """Get a provider by name.

        Args:
            name: Provider name.

        Returns:
            dict[str, Any]: Provider data dict.
        """
        resp = self._stub.GetProvider(
            openshell_pb2.GetProviderRequest(name=name), timeout=self._timeout
        )
        return _provider_to_dict(resp.provider)

    def create(
        self,
        *,
        name: str,
        provider_type: str,
        credentials: dict[str, str] | None = None,
        config: dict[str, str] | None = None,
    ) -> dict[str, Any]:
        """Create a new provider.

        Args:
            name: Provider name.
            provider_type: Provider type identifier.
            credentials: Provider credential key-value pairs.
            config: Provider configuration key-value pairs.

        Returns:
            dict[str, Any]: Created provider data dict.
        """
        provider = datamodel_pb2.Provider(
            name=name,
            type=provider_type,
            credentials=credentials or {},
            config=config or {},
        )
        resp = self._stub.CreateProvider(
            openshell_pb2.CreateProviderRequest(provider=provider),
            timeout=self._timeout,
        )
        return _provider_to_dict(resp.provider)

    def update(
        self,
        *,
        name: str,
        provider_type: str = "",
        credentials: dict[str, str] | None = None,
        config: dict[str, str] | None = None,
    ) -> dict[str, Any]:
        """Update an existing provider.

        Args:
            name: Provider name.
            provider_type: Provider type identifier.
            credentials: Provider credential key-value pairs.
            config: Provider configuration key-value pairs.

        Returns:
            dict[str, Any]: Updated provider data dict.
        """
        provider = datamodel_pb2.Provider(
            name=name,
            type=provider_type,
            credentials=credentials or {},
            config=config or {},
        )
        resp = self._stub.UpdateProvider(
            openshell_pb2.UpdateProviderRequest(provider=provider),
            timeout=self._timeout,
        )
        return _provider_to_dict(resp.provider)

    def delete(self, name: str) -> bool:
        """Delete a provider by name.

        Args:
            name: Provider name.

        Returns:
            bool: True if the provider was deleted.
        """
        resp = self._stub.DeleteProvider(
            openshell_pb2.DeleteProviderRequest(name=name), timeout=self._timeout
        )
        return bool(resp.deleted)



"""REST endpoints for provider CRUD."""

from __future__ import annotations

import asyncio
import logging
from typing import Any

from fastapi import APIRouter, Depends, Query, Request
from pydantic import BaseModel

from shoreguard.api.auth import require_role
from shoreguard.api.deps import _current_gateway, get_actor, get_client
from shoreguard.client import ShoreGuardClient
from shoreguard.services.audit import audit_log
from shoreguard.services.providers import ProviderService

logger = logging.getLogger(__name__)

router = APIRouter()


def _get_provider_service(client: ShoreGuardClient = Depends(get_client)) -> ProviderService:
    """Build a ProviderService from the injected client.

    Args:
        client: gRPC client for the active gateway.

    Returns:
        ProviderService: Service instance bound to the client.
    """
    return ProviderService(client)


class CreateProviderRequest(BaseModel):
    """Body for creating a new provider.

    Attributes:
        name: Provider name.
        type: Provider type identifier.
        api_key: Optional API key for the provider.
        credentials: Extra credential key-value pairs.
        config: Additional configuration key-value pairs.
    """

    name: str
    type: str
    api_key: str = ""
    credentials: dict[str, str] = {}
    config: dict[str, str] = {}


class UpdateProviderRequest(BaseModel):
    """Body for updating a provider.

    Attributes:
        type: Provider type identifier.
        credentials: Credential key-value pairs to update.
        config: Configuration key-value pairs to update.
    """

    type: str = ""
    credentials: dict[str, str] = {}
    config: dict[str, str] = {}


@router.get("/types")
async def list_provider_types() -> list[dict[str, str]]:
    """List known provider types with metadata (label, icon, cred_key).

    Returns:
        list[dict[str, str]]: Provider type definitions.
    """
    return ProviderService.list_known_types()


@router.get("/inference-providers")
async def list_inference_providers() -> list[dict[str, str]]:
    """List known inference provider options.

    Returns:
        list[dict[str, str]]: Inference provider definitions.
    """
    return ProviderService.list_inference_providers()


@router.get("/community-sandboxes")
async def list_community_sandboxes() -> list[dict[str, Any]]:
    """List community sandbox templates from openshell.yaml.

    Returns:
        list[dict[str, Any]]: Community sandbox template definitions.
    """
    return ProviderService.list_community_sandboxes()


@router.get("")
async def list_providers(
    limit: int = Query(100, ge=1, le=1000),
    offset: int = Query(0, ge=0),
    svc: ProviderService = Depends(_get_provider_service),
) -> list[dict[str, Any]]:
    """List all providers.

    Args:
        limit: Maximum number of results to return.
        offset: Number of results to skip.
        svc: Injected provider service.

    Returns:
        list[dict[str, Any]]: Provider records.
    """
    return await asyncio.to_thread(svc.list, limit=limit, offset=offset)


@router.post(
    "",
    status_code=201,
    dependencies=[Depends(require_role("operator"))],
)
async def create_provider(
    body: CreateProviderRequest,
    request: Request,
    svc: ProviderService = Depends(_get_provider_service),
) -> dict[str, Any]:
    """Create a new provider.

    Args:
        body: Provider creation payload.
        request: Incoming HTTP request.
        svc: Injected provider service.

    Returns:
        dict[str, Any]: Created provider record.
    """
    result = await asyncio.to_thread(
        svc.create,
        name=body.name,
        provider_type=body.type,
        api_key=body.api_key,
        extra_credentials=body.credentials or None,
        config=body.config or None,
    )
    logger.info(
        "Provider created (provider=%s, actor=%s)",
        body.name,
        get_actor(request),
    )
    await audit_log(
        request,
        "provider.create",
        "provider",
        body.name,
        gateway=_current_gateway.get(),
        detail={"type": body.type},
    )
    return result


@router.get("/{name}")
async def get_provider(
    name: str,
    svc: ProviderService = Depends(_get_provider_service),
) -> dict[str, Any]:
    """Get a provider by name.

    Args:
        name: Provider name.
        svc: Injected provider service.

    Returns:
        dict[str, Any]: Provider record.
    """
    return await asyncio.to_thread(svc.get, name)


@router.put(
    "/{name}",
    dependencies=[Depends(require_role("operator"))],
)
async def update_provider(
    name: str,
    body: UpdateProviderRequest,
    request: Request,
    svc: ProviderService = Depends(_get_provider_service),
) -> dict[str, Any]:
    """Update an existing provider.

    Args:
        name: Provider name.
        body: Provider update payload.
        request: Incoming HTTP request.
        svc: Injected provider service.

    Returns:
        dict[str, Any]: Updated provider record.
    """
    result = await asyncio.to_thread(
        svc.update,
        name=name,
        provider_type=body.type,
        credentials=body.credentials or None,
        config=body.config or None,
    )
    logger.info(
        "Provider updated (provider=%s, actor=%s)",
        name,
        get_actor(request),
    )
    await audit_log(request, "provider.update", "provider", name, gateway=_current_gateway.get())
    return result


@router.delete(
    "/{name}",
    dependencies=[Depends(require_role("operator"))],
)
async def delete_provider(
    name: str,
    request: Request,
    svc: ProviderService = Depends(_get_provider_service),
) -> dict[str, bool]:
    """Delete a provider by name.

    Args:
        name: Provider name.
        request: Incoming HTTP request.
        svc: Injected provider service.

    Returns:
        dict[str, bool]: Deletion status.
    """
    deleted = await asyncio.to_thread(svc.delete, name)
    if deleted:
        logger.info(
            "Provider deleted (provider=%s, actor=%s)",
            name,
            get_actor(request),
        )
        await audit_log(
            request, "provider.delete", "provider", name, gateway=_current_gateway.get()
        )
    return {"deleted": deleted}



"""Add description and labels_json columns to gateways table.

Revision ID: 004
Revises: 003
Create Date: 2026-04-04
"""

from collections.abc import Sequence

import sqlalchemy as sa
from alembic import op

revision: str = "004"
down_revision: str | None = "003"
branch_labels: str | Sequence[str] | None = None
depends_on: str | Sequence[str] | None = None


def upgrade() -> None:
    """Add description and labels columns to gateways table."""
    op.add_column(
        "gateways",
        sa.Column("description", sa.Text(), nullable=True),
    )
    op.add_column(
        "gateways",
        sa.Column("labels_json", sa.Text(), nullable=True),
    )


def downgrade() -> None:
    """Remove description and labels columns from gateways table."""
    op.drop_column("gateways", "labels_json")
    op.drop_column("gateways", "description")



"""Policy management operations."""

from __future__ import annotations

from typing import Any

from ._proto import openshell_pb2, openshell_pb2_grpc, sandbox_pb2

POLICY_STATUS_NAMES = {
    0: "unspecified",
    1: "pending",
    2: "loaded",
    3: "failed",
    4: "superseded",
}


def _policy_to_dict(policy: sandbox_pb2.SandboxPolicy) -> dict[str, Any]:
    """Convert a SandboxPolicy protobuf to a plain dict.

    Args:
        policy: SandboxPolicy protobuf message.

    Returns:
        dict[str, Any]: Policy data with version, filesystem, process,
            landlock, and network_policies.
    """
    result: dict[str, Any] = {"version": policy.version}

    if policy.HasField("filesystem"):
        result["filesystem"] = {
            "include_workdir": policy.filesystem.include_workdir,
            "read_only": list(policy.filesystem.read_only),
            "read_write": list(policy.filesystem.read_write),
        }
    if policy.HasField("process"):
        result["process"] = {
            "run_as_user": policy.process.run_as_user,
            "run_as_group": policy.process.run_as_group,
        }
    if policy.HasField("landlock"):
        result["landlock"] = {"compatibility": policy.landlock.compatibility}

    network_policies: dict[str, Any] = {}
    for key, rule in policy.network_policies.items():
        network_policies[key] = _network_rule_to_dict(rule)
    if network_policies:
        result["network_policies"] = network_policies
    return result


def _network_rule_to_dict(rule: sandbox_pb2.NetworkPolicyRule) -> dict[str, Any]:
    """Convert a NetworkPolicyRule protobuf to dict.

    Args:
        rule: NetworkPolicyRule protobuf message.

    Returns:
        dict[str, Any]: Network rule data with name, endpoints,
            and binaries.
    """
    result: dict[str, Any] = {"name": rule.name, "endpoints": [], "binaries": []}
    for ep in rule.endpoints:
        ep_dict: dict[str, Any] = {"host": ep.host, "port": ep.port}
        if ep.protocol:
            ep_dict["protocol"] = ep.protocol
        if ep.tls:
            ep_dict["tls"] = ep.tls
        if ep.enforcement:
            ep_dict["enforcement"] = ep.enforcement
        if ep.access:
            ep_dict["access"] = ep.access
        if ep.rules:
            rules_list = []
            for r in ep.rules:
                allow_dict: dict[str, Any] = {
                    "method": r.allow.method,
                    "path": r.allow.path,
                    "command": r.allow.command,
                }
                if r.allow.query:
                    allow_dict["query"] = {
                        key: {
                            **({"glob": m.glob} if m.glob else {}),
                            **({"any": list(m.any)} if m.any else {}),
                        }
                        for key, m in r.allow.query.items()
                    }
                rules_list.append({"allow": allow_dict})
            ep_dict["rules"] = rules_list
        if ep.allowed_ips:
            ep_dict["allowed_ips"] = list(ep.allowed_ips)
        if ep.ports:
            ep_dict["ports"] = list(ep.ports)
        result["endpoints"].append(ep_dict)
    for binary in rule.binaries:
        result["binaries"].append({"path": binary.path})
    return result


class PolicyManager:
    """Policy read/write operations against OpenShell gateway.

    Args:
        stub: OpenShell gRPC stub.
        timeout: gRPC call timeout in seconds.
    """

    def __init__(self, stub: openshell_pb2_grpc.OpenShellStub, *, timeout: float = 30.0) -> None:  # noqa: D107
        self._stub = stub
        self._timeout = timeout

    def get(self, sandbox_name: str) -> dict[str, Any]:
        """Get the current active policy for a sandbox.

        Args:
            sandbox_name: Sandbox name.

        Returns:
            dict[str, Any]: Active policy status with revision details.
        """
        resp = self._stub.GetSandboxPolicyStatus(
            openshell_pb2.GetSandboxPolicyStatusRequest(name=sandbox_name),
            timeout=self._timeout,
        )
        result: dict[str, Any] = {
            "active_version": resp.active_version,
            "revision": {
                "version": resp.revision.version,
                "status": POLICY_STATUS_NAMES.get(resp.revision.status, "unknown"),
                "policy_hash": resp.revision.policy_hash,
                "created_at_ms": resp.revision.created_at_ms,
                "loaded_at_ms": resp.revision.loaded_at_ms,
            },
        }
        if resp.revision.HasField("policy"):
            result["policy"] = _policy_to_dict(resp.revision.policy)
        return result

    def get_version(self, sandbox_name: str, version: int) -> dict[str, Any]:
        """Get a specific policy revision by version number.

        Args:
            sandbox_name: Sandbox name.
            version: Policy version number to retrieve.

        Returns:
            dict[str, Any]: Policy status with revision details.
        """
        resp = self._stub.GetSandboxPolicyStatus(
            openshell_pb2.GetSandboxPolicyStatusRequest(name=sandbox_name, version=version),
            timeout=self._timeout,
        )
        result: dict[str, Any] = {
            "active_version": resp.active_version,
            "revision": {
                "version": resp.revision.version,
                "status": POLICY_STATUS_NAMES.get(resp.revision.status, "unknown"),
                "policy_hash": resp.revision.policy_hash,
                "created_at_ms": resp.revision.created_at_ms,
                "loaded_at_ms": resp.revision.loaded_at_ms,
            },
        }
        if resp.revision.HasField("policy"):
            result["policy"] = _policy_to_dict(resp.revision.policy)
        return result

    def list_revisions(
        self, sandbox_name: str, *, limit: int = 20, offset: int = 0
    ) -> list[dict[str, Any]]:
        """List policy revision history for a sandbox.

        Args:
            sandbox_name: Sandbox name.
            limit: Maximum number of revisions to return.
            offset: Pagination offset.

        Returns:
            list[dict[str, Any]]: List of policy revision summary dicts.
        """
        resp = self._stub.ListSandboxPolicies(
            openshell_pb2.ListSandboxPoliciesRequest(name=sandbox_name, limit=limit, offset=offset),
            timeout=self._timeout,
        )
        return [
            {
                "version": rev.version,
                "status": POLICY_STATUS_NAMES.get(rev.status, "unknown"),
                "policy_hash": rev.policy_hash,
                "created_at_ms": rev.created_at_ms,
                "loaded_at_ms": rev.loaded_at_ms,
                "load_error": rev.load_error,
            }
            for rev in resp.revisions
        ]

    def update(
        self, sandbox_name: str, policy: sandbox_pb2.SandboxPolicy, *, global_scope: bool = False
    ) -> dict[str, Any]:
        """Push a new policy version to a sandbox (or globally).

        Args:
            sandbox_name: Sandbox name.
            policy: SandboxPolicy protobuf message.
            global_scope: If True, apply policy globally.

        Returns:
            dict[str, Any]: Version and policy hash of the new revision.
        """
        resp = self._stub.UpdateConfig(
            openshell_pb2.UpdateConfigRequest(
                name=sandbox_name,
                policy=policy,
                **{"global": global_scope},  # type: ignore[arg-type]
            ),
            timeout=self._timeout,
        )
        return {
            "version": resp.version,
            "policy_hash": resp.policy_hash,
        }



"""Message formatters for notification channels (Slack, Discord, Email, Generic)."""

from __future__ import annotations

import json
from typing import Any

_EVENT_LABELS: dict[str, str] = {
    "approval.pending": "Approval Pending",
    "approval.approved": "Approval Approved",
    "approval.rejected": "Approval Rejected",
    "sandbox.created": "Sandbox Created",
    "sandbox.deleted": "Sandbox Deleted",
    "gateway.registered": "Gateway Registered",
    "gateway.unregistered": "Gateway Unregistered",
    "inference.updated": "Inference Updated",
    "policy.updated": "Policy Updated",
    "webhook.test": "Test Event",
}

_SLACK_COLORS: dict[str, str] = {
    "approval.pending": "warning",
    "approval.approved": "good",
    "approval.rejected": "danger",
    "sandbox.created": "#2196F3",
    "sandbox.deleted": "#9E9E9E",
    "gateway.registered": "good",
    "gateway.unregistered": "warning",
    "inference.updated": "#2196F3",
    "policy.updated": "#2196F3",
    "webhook.test": "#6C757D",
}

_DISCORD_COLORS: dict[str, int] = {
    "approval.pending": 0xFFA500,
    "approval.approved": 0x2ECC71,
    "approval.rejected": 0xE74C3C,
    "sandbox.created": 0x2196F3,
    "sandbox.deleted": 0x9E9E9E,
    "gateway.registered": 0x2ECC71,
    "gateway.unregistered": 0xFFA500,
    "inference.updated": 0x2196F3,
    "policy.updated": 0x2196F3,
    "webhook.test": 0x6C757D,
}


def _event_label(event_type: str) -> str:
    """Return a human-readable label for an event type.

    Args:
        event_type: Machine-readable event type string.

    Returns:
        str: Human-readable label.
    """
    return _EVENT_LABELS.get(event_type, event_type.replace(".", " ").title())


def _payload_fields(payload: dict[str, Any]) -> list[tuple[str, str]]:
    """Extract key-value pairs from a payload for display.

    Args:
        payload: Event data payload.

    Returns:
        list[tuple[str, str]]: List of (label, value) pairs.
    """
    fields = []
    for key in (
        "sandbox",
        "gateway",
        "actor",
        "reason",
        "message",
        "provider",
        "model",
        "image",
        "endpoint",
    ):
        if key in payload:
            fields.append((key.title(), str(payload[key])))
    return fields


def format_generic(event_type: str, payload: dict[str, Any], timestamp: str) -> str:
    """Format a generic webhook payload (JSON envelope with HMAC signing).

    Args:
        event_type: Machine-readable event type.
        payload: Event data payload.
        timestamp: ISO-8601 timestamp string.

    Returns:
        str: JSON-encoded payload string.
    """
    return json.dumps(
        {"event": event_type, "timestamp": timestamp, "data": payload},
        default=str,
    )


def format_slack(event_type: str, payload: dict[str, Any], timestamp: str) -> str:
    """Format a Slack Block Kit message.

    Args:
        event_type: Machine-readable event type.
        payload: Event data payload.
        timestamp: ISO-8601 timestamp string.

    Returns:
        str: JSON-encoded Slack payload.
    """
    label = _event_label(event_type)
    color = _SLACK_COLORS.get(event_type, "#6C757D")
    fields = _payload_fields(payload)
    field_text = "\n".join(f"*{k}:* {v}" for k, v in fields) if fields else ""

    attachment = {
        "color": color,
        "blocks": [
            {
                "type": "section",
                "text": {"type": "mrkdwn", "text": f"*{label}*\n{field_text}"},
            },
            {
                "type": "context",
                "elements": [{"type": "mrkdwn", "text": f"ShoreGuard | {timestamp}"}],
            },
        ],
    }
    return json.dumps({"attachments": [attachment]})


def format_discord(event_type: str, payload: dict[str, Any], timestamp: str) -> str:
    """Format a Discord embed message.

    Args:
        event_type: Machine-readable event type.
        payload: Event data payload.
        timestamp: ISO-8601 timestamp string.

    Returns:
        str: JSON-encoded Discord payload.
    """
    label = _event_label(event_type)
    color = _DISCORD_COLORS.get(event_type, 0x6C757D)
    fields = [{"name": k, "value": v, "inline": True} for k, v in _payload_fields(payload)]

    embed: dict[str, Any] = {
        "title": label,
        "color": color,
        "timestamp": timestamp,
        "footer": {"text": "ShoreGuard"},
    }
    if fields:
        embed["fields"] = fields

    return json.dumps({"embeds": [embed]})


def format_email_body(event_type: str, payload: dict[str, Any], timestamp: str) -> str:
    """Format a plain-text email body.

    Args:
        event_type: Machine-readable event type.
        payload: Event data payload.
        timestamp: ISO-8601 timestamp string.

    Returns:
        str: Plain-text email body.
    """
    label = _event_label(event_type)
    lines = [f"ShoreGuard — {label}", f"Time: {timestamp}", ""]
    for key, value in _payload_fields(payload):
        lines.append(f"{key}: {value}")
    return "\n".join(lines)


FORMATTERS: dict[str, Any] = {
    "generic": format_generic,
    "slack": format_slack,
    "discord": format_discord,
    "email": format_email_body,
}



"""Prometheus metrics endpoint and HTTP request middleware."""

from __future__ import annotations

import asyncio

from fastapi import APIRouter, Request, Response
from prometheus_client import CONTENT_TYPE_LATEST, Counter, Gauge, Info, generate_latest

# ── Metrics definitions ──────────────────────────────────────────────────

shoreguard_info = Info("shoreguard", "ShoreGuard build information")

gateways_total = Gauge(
    "shoreguard_gateways_total",
    "Number of registered gateways by status",
    ["status"],
)

operations_total = Gauge(
    "shoreguard_operations_total",
    "Number of tracked operations by status",
    ["status"],
)

webhook_deliveries_total = Counter(
    "shoreguard_webhook_deliveries_total",
    "Total webhook delivery attempts by result",
    ["status"],
)

http_requests_total = Counter(
    "shoreguard_http_requests_total",
    "Total HTTP requests by method and status code",
    ["method", "status"],
)

# ── Router ───────────────────────────────────────────────────────────────

router = APIRouter(tags=["metrics"])


@router.get("/metrics")
async def metrics() -> Response:
    """Prometheus metrics endpoint (unauthenticated).

    Returns:
        Response: Prometheus text format metrics.
    """
    await asyncio.to_thread(_collect_gauges)
    return Response(content=generate_latest(), media_type=CONTENT_TYPE_LATEST)


# ── Middleware ────────────────────────────────────────────────────────────


async def metrics_middleware(request: Request, call_next: object) -> Response:
    """Count HTTP requests by method and status code.

    Args:
        request: The incoming HTTP request.
        call_next: ASGI middleware chain.

    Returns:
        Response: The response from the next middleware/handler.
    """
    response = await call_next(request)  # type: ignore[operator]
    if not request.url.path.startswith("/metrics"):
        http_requests_total.labels(
            method=request.method,
            status=str(response.status_code),
        ).inc()
    return response  # type: ignore[return-value]


# ── Gauge collection ─────────────────────────────────────────────────────


def _collect_gauges() -> None:
    """Update gauge values from current service state."""
    import shoreguard.services.gateway as gw_mod
    from shoreguard.services.operations import operation_store

    # Gateway status counts
    if gw_mod.gateway_service is not None:
        all_gw = gw_mod.gateway_service.list_all()
        counts: dict[str, int] = {}
        for g in all_gw:
            status = g.get("status", "unknown")
            counts[status] = counts.get(status, 0) + 1
        gateways_total._metrics.clear()
        for status, count in counts.items():
            gateways_total.labels(status=status).set(count)

    # Operation status counts
    op_counts = operation_store.status_counts()
    operations_total._metrics.clear()
    for status, count in op_counts.items():
        operations_total.labels(status=status).set(count)
